# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from datetime import datetime
from time import struct_time
from unittest import mock
from unittest.mock import call, MagicMock

from superdesk.tests import TestCase
from superdesk.io.commands.update_ingest import LAST_ITEM_UPDATE

feed_parse = MagicMock()
requests_get = MagicMock()

nrk_xml = """
<rss version="2.0">
<channel>
<description/>
<link>https://www.nrk.no/finnmark/siste-nytt-finnmark-1.11957920</link>
<pubDate>Mon, 06 Feb 2017 10:51:00 +0100</pubDate>
<item>
    <guid isPermaLink="false">1.13362571</guid>
    <title>Stanset ikke for fotgjenger</title>
    <description>
    En fører av en personbil er anmeldt for ikke...
    </description>
    <link>
    https://www.nrk.no/finnmark/stanset-ikke-for-fotgjenger-1.13362571
    </link>
    <pubDate>Mon, 06 Feb 2017 10:51:00 +0100</pubDate>
</item>
</channel>
</rss>
"""


class RssResponse:
    ok = True
    content = nrk_xml


class RssError(Exception):
    def __init__(self, name, orig_ex, provider):
        self.name = name
        self.orig_ex = orig_ex
        self.provider = provider


class FakeIngestApiError(Exception):
    """Mocked factory for ingest API errors."""

    @classmethod
    def apiGeneralError(cls, exception, provider):
        return RssError("general_error", exception, provider)

    @classmethod
    def apiNotFoundError(cls, exception, provider):
        return RssError("not_found_error", exception, provider)

    @classmethod
    def apiAuthError(cls, exception, provider):
        return RssError("auth_error", exception, provider)


class FakeParseError(Exception):
    @classmethod
    def parseMessageError(cls, exception=None, provider=None, data=None):
        return FakeParseError(exception)


class RssIngestServiceTest(TestCase):
    """Base class for RSSFeedingService tests."""

    def setUpForChildren(self):
        super().setUpForChildren()
        try:
            from superdesk.io.feeding_services.rss import RSSFeedingService
        except ImportError:
            # a missing class should result in a test failure, not in an error
            self.fail("Could not import class under test (RSSFeedingService).")
        else:
            self.instance = RSSFeedingService()
            self.config = {}
            self.instance.provider = {"config": self.config}


class InstanceInitTestCase(RssIngestServiceTest):
    """Tests if instance is correctly initialized on creation."""

    def test_initializes_auth_info_to_none(self):
        self.assertIsNone(self.instance.auth_info)


class PrepareHrefMethodTestCase(RssIngestServiceTest):
    """Tests for the prepare_href() method."""

    def test_returns_unchanged_link_if_no_auth_info(self):
        url = "http://domain.com/images/foo.jpg"

        returned = self.instance.prepare_href(url)

        self.assertEqual(returned, url)

    def test_returns_link_with_auth_data_if_available(self):
        self.instance.provider["config"].update({"username": "john doe", "password": "abc+007"})
        url = "http://domain.com/images/foo.jpg"

        returned = self.instance.prepare_href(url)

        self.assertEqual(returned, "http://john%20doe:abc%2B007@domain.com/images/foo.jpg")


@mock.patch("superdesk.io.feeding_services.rss.feedparser.parse", feed_parse)
@mock.patch("superdesk.io.feeding_services.http_base_service.IngestApiError", FakeIngestApiError)
@mock.patch("superdesk.io.feeding_services.rss.ParserError", FakeParseError)
class UpdateMethodTestCase(RssIngestServiceTest):
    """Tests for the _update() method."""

    def _hard_reset_mock(self, mock):
        """Reset a mock and also clear its return value and side effects.

        :param MagicMock mock: mock instance to reset
        """
        mock.reset_mock()
        mock.return_value = MagicMock(name="mock()")
        mock.side_effect = None

    def setUp(self):
        self._hard_reset_mock(feed_parse)
        self.instance._fetch_data = MagicMock(return_value="<rss>foo</rss>")
        self.instance._create_item = MagicMock(return_value={})
        self.instance._extract_image_links = MagicMock(return_value=[])
        self.instance._fetch_images = MagicMock()
        self.instance._wrap_into_package = MagicMock()

    def test_raises_ingest_error_if_fetching_data_fails(self):
        self.instance._fetch_data.side_effect = FakeIngestApiError
        try:
            with self.assertRaises(FakeIngestApiError):
                self.instance._update({}, {})
        except Exception:
            self.fail("Expected exception type was not raised.")

    def test_raises_ingest_error_on_parse_error(self):
        feed_parse.side_effect = Exception("Parse error!")
        with self.assertRaises(FakeParseError):
            self.instance._update({}, {})

    def test_returns_items_built_from_retrieved_data(self):
        feed_parse.return_value = MagicMock(
            entries=[
                MagicMock(updated_parsed=struct_time([2015, 2, 25, 17, 11, 11, 2, 56, 0])),
                MagicMock(updated_parsed=struct_time([2015, 2, 25, 17, 22, 22, 2, 56, 0])),
            ]
        )

        item_1 = dict(
            guid="item_1",
            firstcreated=datetime(2015, 2, 25, 17, 11, 11),
            versioncreated=datetime(2015, 2, 25, 17, 11, 11),
        )
        item_2 = dict(
            guid="item_2",
            firstcreated=datetime(2015, 2, 25, 17, 22, 22),
            versioncreated=datetime(2015, 2, 25, 17, 22, 22),
        )
        self.instance._create_item.side_effect = [item_1, item_2]

        returned = self.instance._update({"last_updated": datetime(2015, 2, 25, 14, 0, 0)}, {})

        self.assertEqual(len(returned), 1)
        items = returned[0]
        self.assertEqual(items, [item_1, item_2])

    def test_does_not_return_old_items(self):
        feed_parse.return_value = MagicMock(
            entries=[
                MagicMock(updated_parsed=struct_time([2015, 2, 25, 11, 11, 11, 2, 56, 0])),
            ]
        )

        item = dict(
            guid="item_1",
            firstcreated=datetime(2015, 2, 25, 11, 11, 11),
            versioncreated=datetime(2015, 2, 25, 11, 11, 11),
        )
        self.instance._create_item.return_value = item

        returned = self.instance._update({LAST_ITEM_UPDATE: datetime(2015, 2, 25, 15, 0, 0)}, {})

        self.assertEqual(len(returned), 1)
        items = returned[0]
        self.assertEqual(len(items), 0)

    def test_returns_items_and_package_list_if_entry_contains_image_urls(self):
        feed_parse.return_value = MagicMock(
            entries=[
                MagicMock(updated_parsed=struct_time([2015, 2, 25, 17, 11, 11, 0, 0, 0])),
            ]
        )

        fake_text_item = dict(
            guid="fake_text_item",
            firstcreated=datetime(2015, 2, 25, 17, 11, 11),
            versioncreated=datetime(2015, 2, 25, 17, 11, 11),
        )
        item_url = "http://link.to/fake_item"
        fake_image_items = [
            {"guid": "http://link.to/image_1.jpg"},
            {"guid": "http://link.to/image_2.jpg"},
        ]
        fake_package = MagicMock(name="fake_package")

        self.instance._create_item = MagicMock(return_value=fake_text_item)
        self.instance._extract_image_links = MagicMock(return_value=[item_url])
        self.instance._create_image_items = MagicMock(return_value=fake_image_items)
        self.instance._create_package = MagicMock(return_value=fake_package)

        returned = self.instance._update({"last_updated": datetime(1965, 2, 24)}, {})

        self.instance._create_package.assert_called_with(fake_text_item, fake_image_items)

        self.assertEqual(len(returned), 1)
        items = returned[0]

        expected_items = [
            fake_text_item,
            {"guid": "http://link.to/image_1.jpg"},
            {"guid": "http://link.to/image_2.jpg"},
            fake_package,
        ]
        self.assertCountEqual(items, expected_items)


@mock.patch("superdesk.io.feeding_services.http_base_service.requests.get", requests_get)
@mock.patch("superdesk.io.feeding_services.http_base_service.IngestApiError", FakeIngestApiError)
class FetchDataMethodTestCase(RssIngestServiceTest):
    """Tests for the _fetch_data() method."""

    def setUp(self):
        requests_get.reset_mock()
        self.fake_provider = MagicMock(name="fake provider")
        self.instance.provider = self.fake_provider
        self.fake_provider.__getitem__.return_value = self.config
        self.fake_provider.setdefault.return_value = self.config

    def test_retrieves_feed_from_correct_url(self):
        requests_get.return_value = MagicMock(ok=True)
        self.config.update(dict(url="http://news.com/rss"))

        self.instance._fetch_data()

        call_args = requests_get.call_args[0]
        self.assertEqual(call_args[0], "http://news.com/rss")

    def test_stores_auth_info_in_instance_if_auth_required(self):
        self.fake_provider["config"].update(
            {
                "url": "foo",
                "auth_required": True,
                "username": "james",
                "password": "bond+007",
            }
        )

        requests_get.return_value = MagicMock(ok=False)

        try:
            self.instance._update(self.fake_provider, {})
        except Exception:
            pass

        self.assertEqual(self.instance.auth_info, {"username": "james", "password": "bond+007"})

    def test_provides_auth_info_if_required(self):
        requests_get.return_value = MagicMock(ok=True)
        self.config.update(dict(url="http://news.com/rss", auth_required=True, username="johndoe", password="secret"))

        self.instance._fetch_data()

        kw_call_args = requests_get.call_args[1]
        self.assertEqual(kw_call_args.get("auth"), ("johndoe", "secret"))

    def test_returns_fetched_data_on_success(self):
        requests_get.return_value = MagicMock(ok=True, content="<rss>X</rss>")
        self.config.update(dict(url="http://news.com/rss"))

        response = self.instance._fetch_data()

        self.assertEqual(response, "<rss>X</rss>")

    def test_raises_auth_error_on_401(self):
        requests_get.return_value = MagicMock(ok=False, status_code=401, reason="invalid credentials")
        self.config.update(dict(url="http://news.com/rss"))

        try:
            with self.assertRaises(RssError) as exc_context:
                self.instance._fetch_data()
        except Exception:
            self.fail("Expected exception type was not raised.")

        ex = exc_context.exception
        self.assertEqual(ex.name, "auth_error")
        self.assertEqual(ex.orig_ex.args[0], "invalid credentials")
        self.assertIs(ex.provider, self.fake_provider)

    def test_raises_auth_error_on_403(self):
        requests_get.return_value = MagicMock(ok=False, status_code=403, reason="access forbidden")
        self.config.update(dict(url="http://news.com/rss"))

        try:
            with self.assertRaises(RssError) as exc_context:
                self.instance._fetch_data()
        except Exception:
            self.fail("Expected exception type was not raised.")

        ex = exc_context.exception
        self.assertEqual(ex.name, "auth_error")
        self.assertEqual(ex.orig_ex.args[0], "access forbidden")
        self.assertIs(ex.provider, self.fake_provider)

    def test_raises_not_found_error_on_404(self):
        requests_get.return_value = MagicMock(ok=False, status_code=404, reason="resource not found")
        self.config.update(dict(url="http://news.com/rss"))

        try:
            with self.assertRaises(RssError) as exc_context:
                self.instance._fetch_data()
        except Exception:
            self.fail("Expected exception type was not raised.")

        ex = exc_context.exception
        self.assertEqual(ex.name, "not_found_error")
        self.assertEqual(ex.orig_ex.args[0], "resource not found")
        self.assertIs(ex.provider, self.fake_provider)

    def test_raises_general_error_on_unknown_error(self):
        requests_get.return_value = MagicMock(ok=False, status_code=500, reason="server down")
        self.config.update(dict(url="http://news.com/rss"))

        try:
            with self.assertRaises(RssError) as exc_context:
                self.instance._fetch_data()
        except Exception:
            self.fail("Expected exception type was not raised.")

        ex = exc_context.exception
        self.assertEqual(ex.name, "general_error")
        self.assertEqual(ex.orig_ex.args[0], "server down")
        self.assertIs(ex.provider, self.fake_provider)


class ExtractImageLinksMethodTestCase(RssIngestServiceTest):
    """Tests for the _extract_image_links() method."""

    def test_extracts_enclosure_img_links(self):
        rss_entry = MagicMock()
        rss_entry.links = [
            {
                "rel": "enclosure",
                "href": "http://foo.bar/image_1.jpg",
                "type": "image/jpeg",
            },
            {
                "rel": "enclosure",
                "href": "http://foo.bar/image_2.png",
                "type": "image/png",
            },
        ]

        links = self.instance._extract_image_links(rss_entry)

        self.assertCountEqual(links, ["http://foo.bar/image_1.jpg", "http://foo.bar/image_2.png"])

    def test_omits_enclosure_links_to_non_supported_mime_types(self):
        rss_entry = MagicMock()
        rss_entry.links = [
            {
                "rel": "alternative",
                "href": "http://foo.bar/81fecd",
                "type": "text/html",
            },
            {
                "rel": "enclosure",
                "href": "http://foo.bar/image_1.bmp",
                "type": "image/bmp",
            },
        ]

        links = self.instance._extract_image_links(rss_entry)

        self.assertCountEqual(links, [])

    def test_extracts_media_thumbnail_links(self):
        rss_entry = MagicMock()
        rss_entry.media_thumbnail = [
            {"url": "http://foo.bar/small_img.gif"},
            {"url": "http://foo.bar/thumb_x.tiff"},
        ]

        links = self.instance._extract_image_links(rss_entry)

        self.assertCountEqual(links, ["http://foo.bar/small_img.gif", "http://foo.bar/thumb_x.tiff"])

    def test_omits_media_thumbnail_links_to_non_supported_formats(self):
        rss_entry = MagicMock()
        rss_entry.media_thumbnail = [
            {"url": "http://foo.bar/image.bmp"},
        ]

        links = self.instance._extract_image_links(rss_entry)

        self.assertCountEqual(links, [])

    def test_extracts_media_content_img_links(self):
        rss_entry = MagicMock()
        rss_entry.media_content = [
            {
                "url": "http://foo.bar/image_1.jpeg",
                "type": "image/jpeg",
            },
            {
                "url": "http://foo.bar/image_2.tiff",
                "type": "image/tiff",
            },
        ]

        links = self.instance._extract_image_links(rss_entry)

        self.assertCountEqual(links, ["http://foo.bar/image_1.jpeg", "http://foo.bar/image_2.tiff"])

    def test_omits_media_content_links_to_non_supported_mime_types(self):
        rss_entry = MagicMock()
        rss_entry.media_content = [
            {
                "url": "http://foo.bar/music.mp3",
                "type": "audio/mpeg3",
            },
            {
                "url": "http://foo.bar/video.avi",
                "type": "video/avi",
            },
            {
                "url": "http://foo.bar/image_1.bmp",
                "type": "image/bmp",
            },
        ]

        links = self.instance._extract_image_links(rss_entry)

        self.assertCountEqual(links, [])

    def test_omits_duplicate_links(self):
        rss_entry = MagicMock()
        rss_entry.links = [
            {
                "rel": "enclosure",
                "href": "http://foo.bar/image.png",
                "type": "image/png",
            },
            {
                "rel": "enclosure",
                "href": "http://foo.bar/image.png",
                "type": "image/png",
            },
        ]
        rss_entry.media_content = [
            {
                "url": "http://foo.bar/image.png",
                "type": "image/png",
            },
            {
                "url": "http://foo.bar/image.png",
                "type": "image/jpeg",
            },
        ]

        links = self.instance._extract_image_links(rss_entry)

        self.assertCountEqual(links, ["http://foo.bar/image.png"])


class CreateItemMethodTestCase(RssIngestServiceTest):
    """Tests for the _create_item() method."""

    def test_creates_item_from_given_data(self):
        data = dict(
            guid="http://news.com/rss/1234abcd",
            published_parsed=struct_time([2015, 2, 25, 16, 45, 23, 2, 56, 0]),
            updated_parsed=struct_time([2015, 2, 25, 17, 52, 11, 2, 56, 0]),
            title="Breaking News!",
            summary="Something happened...",
            body_text="This is body text.",
            author="author",
        )

        item = self.instance._create_item(data, source="source")

        self.assertEqual(item.get("guid"), "http://news.com/rss/1234abcd")
        self.assertEqual(item.get("uri"), "http://news.com/rss/1234abcd")
        self.assertEqual(item.get("type"), "text")
        self.assertEqual(item.get("firstcreated"), datetime(2015, 2, 25, 16, 45, 23))
        self.assertEqual(item.get("versioncreated"), datetime(2015, 2, 25, 17, 52, 11))
        self.assertEqual(item.get("headline"), "Breaking News!")
        self.assertEqual(item.get("abstract"), "Something happened...")
        self.assertEqual(
            item.get("body_html"),
            '<p><a href="http://news.com/rss/1234abcd" target="_blank">source</a></p>This is body text.',
        )
        self.assertEqual(item.get("byline"), "author")
        dateline = item.get("dateline", {})
        self.assertEqual(dateline.get("source"), "source")
        self.assertEqual(dateline.get("date"), item.get("firstcreated"))

    def test_populates_body_text_from_content_field_as_fallback(self):
        class CustomDict(dict):
            """Customized dict class, allows adding custom attributes to it."""

        data = CustomDict(
            guid="http://news.com/rss/1234abcd",
            published_parsed=struct_time([2015, 2, 25, 16, 45, 23, 2, 56, 0]),
            updated_parsed=struct_time([2015, 2, 25, 17, 52, 11, 2, 56, 0]),
            title="Breaking News!",
            summary="Something happened...",
            # NOTE: no body_text field
        )

        content_field = [CustomDict(type="text/html", value="<p>This is body</p>")]
        content_field[0].value = "<p>This is body</p>"
        data.content = content_field

        item = self.instance._create_item(data)

        self.assertEqual(
            item.get("body_html"),
            '<p><a href="http://news.com/rss/1234abcd" target="_blank">source</a></p><p>This is body</p>',
        )

    def test_does_not_use_body_text_populate_fallback_if_aliased(self):
        class CustomDict(dict):
            """Customized dict class, allows adding custom attributes to it."""

        data = CustomDict(
            guid="http://news.com/rss/1234abcd",
            published_parsed=struct_time([2015, 2, 25, 16, 45, 23, 2, 56, 0]),
            updated_parsed=struct_time([2015, 2, 25, 17, 52, 11, 2, 56, 0]),
            title="Breaking News!",
            summary="Something happened...",
            # NOTE: no body_text field
        )

        content_field = [CustomDict(type="text/html", value="<p>This is body</p>")]
        content_field[0].value = "<p>This is body</p>"
        data.content = content_field

        field_aliases = [{"body_text": "body_text_field_alias"}]
        data.body_text_field_alias = None  # simulate non-existing alias field

        item = self.instance._create_item(data, field_aliases)

        self.assertEqual(
            item.get("body_html"), '<p><a href="http://news.com/rss/1234abcd" target="_blank">source</a></p>'
        )

    def test_creates_item_taking_field_name_aliases_into_account(self):
        data = dict(
            guid="http://news.com/rss/1234abcd",
            published_parsed=struct_time([2015, 2, 25, 16, 45, 23, 2, 56, 0]),
            updated_parsed=struct_time([2015, 2, 25, 17, 52, 11, 2, 56, 0]),
            title_field_alias="Breaking News!",
            summary_field_alias="Something happened...",
            body_text_field_alias="This is body text.",
        )

        field_aliases = [
            {"title": "title_field_alias"},
            {"summary": "summary_field_alias"},
            {"body_text": "body_text_field_alias"},
        ]

        item = self.instance._create_item(data, field_aliases)

        self.assertEqual(item.get("guid"), "http://news.com/rss/1234abcd")
        self.assertEqual(item.get("uri"), "http://news.com/rss/1234abcd")
        self.assertEqual(item.get("type"), "text")
        self.assertEqual(item.get("firstcreated"), datetime(2015, 2, 25, 16, 45, 23))
        self.assertEqual(item.get("versioncreated"), datetime(2015, 2, 25, 17, 52, 11))
        self.assertEqual(item.get("headline"), "Breaking News!")
        self.assertEqual(item.get("abstract"), "Something happened...")
        self.assertEqual(
            item.get("body_html"),
            '<p><a href="http://news.com/rss/1234abcd" target="_blank">source</a></p>This is body text.',
        )

    def test_aliases_fields_are_skipped(self):
        data = dict(
            guid="http://news.com/rss/1234abcd",
            published_parsed=struct_time([2015, 2, 25, 16, 45, 23, 2, 56, 0]),
            updated_parsed=struct_time([2015, 2, 25, 17, 52, 11, 2, 56, 0]),
            title="Breaking News!",
            summary="This is body text.",
        )

        field_aliases = [{"body_text": "summary"}]

        item = self.instance._create_item(data, field_aliases)

        self.assertEqual(item.get("guid"), "http://news.com/rss/1234abcd")
        self.assertEqual(item.get("uri"), "http://news.com/rss/1234abcd")
        self.assertEqual(item.get("type"), "text")
        self.assertEqual(item.get("firstcreated"), datetime(2015, 2, 25, 16, 45, 23))
        self.assertEqual(item.get("versioncreated"), datetime(2015, 2, 25, 17, 52, 11))
        self.assertEqual(item.get("headline"), "Breaking News!")
        self.assertEqual(
            item.get("body_html"),
            '<p><a href="http://news.com/rss/1234abcd" target="_blank">source</a></p>This is body text.',
        )
        self.assertIsNone(item.get("abstract"))  # because summary is aliased

    def test_aliases_fields_are_skipped_unless_themselves_aliased(self):
        data = dict(
            guid="http://news.com/rss/1234abcd",
            published_parsed=struct_time([2015, 2, 25, 16, 45, 23, 2, 56, 0]),
            updated_parsed=struct_time([2015, 2, 25, 17, 52, 11, 2, 56, 0]),
            title="Breaking News!",
            summary="This is body text.",
            link="http://news.com/1234abcd",
        )

        field_aliases = [{"body_text": "summary"}, {"summary": "link"}]

        item = self.instance._create_item(data, field_aliases)

        self.assertEqual(item.get("abstract"), "http://news.com/1234abcd")

    def test_guid_not_permalink(self):
        self.instance.provider = provider = {"config": {"url": "http://example.com/rss"}}
        with mock.patch("superdesk.io.feeding_services.http_base_service.requests.get", return_value=RssResponse()):
            items = self.instance._update(provider, None)[0]
        self.assertEqual(1, len(items))
        self.assertEqual("https://www.nrk.no/finnmark/stanset-ikke-for-fotgjenger-1.13362571", items[0]["uri"])
        self.assertEqual("tag:www.nrk.no:1.13362571", items[0]["guid"])
        self.assertIn(items[0]["uri"], items[0]["body_html"])

    def test_guid_not_set(self):
        self.instance.provider = provider = {"config": {"url": "http://example.com/rss"}}
        response = RssResponse()
        response.content = nrk_xml.replace('<guid isPermaLink="false">1.13362571</guid>', "")
        with mock.patch("superdesk.io.feeding_services.http_base_service.requests.get", return_value=response):
            items = self.instance._update(provider, None)[0]
        self.assertEqual("tag:www.nrk.no:finnmark:stanset-ikke-for-fotgjenger-1.13362571", items[0]["guid"])


class CreateImageItemsMethodTestCase(RssIngestServiceTest):
    """Tests for the _create_image_items() method."""

    def test_creates_image_items_from_given_urls(self):
        links = [
            "http://foo.bar/image_1.jpg",
            "http://baz.ban/image_2.jpg",
        ]
        text_item = {
            "firstcreated": datetime(2015, 1, 27, 15, 56, 59),
            "versioncreated": datetime(2015, 3, 19, 17, 20, 45),
        }

        result = self.instance._create_image_items(links, text_item)

        expected_result = [
            {
                "guid": "tag:foo.bar:image_1.jpg",
                "type": "picture",
                "firstcreated": datetime(2015, 1, 27, 15, 56, 59),
                "versioncreated": datetime(2015, 3, 19, 17, 20, 45),
                "renditions": {"baseImage": {"href": "http://foo.bar/image_1.jpg"}},
            },
            {
                "guid": "tag:baz.ban:image_2.jpg",
                "type": "picture",
                "firstcreated": datetime(2015, 1, 27, 15, 56, 59),
                "versioncreated": datetime(2015, 3, 19, 17, 20, 45),
                "renditions": {"baseImage": {"href": "http://baz.ban/image_2.jpg"}},
            },
        ]
        self.maxDiff = None
        self.assertCountEqual(result, expected_result)


class CreatePackageMethodTestCase(RssIngestServiceTest):
    """Tests for the _create_package() method."""

    def test_creates_package_from_given_text_and_image_items(self):
        text_item = {
            "guid": "main_text",
            "firstcreated": datetime(2015, 1, 27, 16, 0, 0),
            "versioncreated": datetime(2015, 1, 27, 16, 0, 0),
            "headline": "Headline of the text item",
        }
        img_item_1 = {"guid": "image_1"}
        img_item_2 = {"guid": "image_2"}

        package = self.instance._create_package(text_item, [img_item_1, img_item_2])

        expected = {
            "type": "composite",
            "guid": "main_text:pkg",
            "firstcreated": datetime(2015, 1, 27, 16, 0, 0),
            "versioncreated": datetime(2015, 1, 27, 16, 0, 0),
            "headline": "Headline of the text item",
            "groups": [
                {
                    "id": "root",
                    "role": "grpRole:NEP",
                    "refs": [{"idRef": "main"}],
                },
                {
                    "id": "main",
                    "role": "main",
                    "refs": [
                        {"residRef": "main_text"},
                        {"residRef": "image_1"},
                        {"residRef": "image_2"},
                    ],
                },
            ],
        }
        self.assertEqual(package, expected)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk import Resource


class MediaReferencesResource(Resource):
    schema = {
        # item where media belongs i.e. in case of picture item it will be picture item
        # in case of associated featuremedia it will be the parent item id.
        "item_id": {"type": "string", "required": True},
        # media of the renditions
        "media_id": {"type": "string", "required": True},
        # associated item id
        "associated_id": {"type": "string", "nullable": True},
        "published": {"type": "boolean"},
    }
    endpoint_name = "media_references"
    internal_resource = False
    resource_methods = ["GET"]
    item_methods = ["GET"]
    mongo_indexes = {
        "item_id_1_media_1": [("item_id", 1), ("media_id", 1)],
        "media_1": [("media_id", 1)],
        "associated_id": [("associated_id", 1)],
    }
    query_objectid_as_string = True



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from superdesk.publish import publish_service
from superdesk.publish import register_transmitter
from superdesk.errors import PublishFileError
from os import path

errors = [PublishFileError.fileSaveError().get_error_description()]


class FilePublishService(publish_service.PublishService):
    """Superdesk file transmitter.

    It creates files on superdesk server in configured folder.
    """

    NAME = "File"

    def _transmit(self, queue_item, subscriber):
        try:
            config = queue_item["destination"]["config"]
            file_path = config["file_path"]
            if not path.isabs(file_path):
                file_path = "/" + file_path
            with open(path.join(file_path, publish_service.get_publish_service().get_filename(queue_item)), "wb") as f:
                f.write(queue_item["encoded_item"])
        except Exception as ex:
            raise PublishFileError.fileSaveError(ex, config)


register_transmitter("File", FilePublishService(), errors)



import os
import json
import pytest
from pathlib import Path
from bson import ObjectId
from flask import url_for
from requests.auth import _basic_auth_str

from superdesk.tests import get_mongo_uri, setup, clean_dbs
from superdesk.factory import get_app as get_sd_app
from superdesk.auth_server.clients import RegisterClient
from prod_api.app import get_app as get_prodapi_api


MONGO_DB = "prodapi_tests"
ELASTICSEARCH_INDEX = MONGO_DB
AUTH_SERVER_SHARED_SECRET = "2kZOf0VI9T70vU9uMlKLyc5GlabxVgl6"


def get_test_prodapi_app(extra_config=None):
    """
    Create and return configured test prod api flask app.
    :param extra_config: extra settings
    :return: eve.flaskapp.Eve
    """
    test_config = {
        "DEBUG": True,
        "TESTING": True,
        "SUPERDESK_TESTING": True,
        "MONGO_CONNECT": False,
        "MONGO_MAX_POOL_SIZE": 1,
        "MONGO_DBNAME": MONGO_DB,
        "MONGO_URI": get_mongo_uri("MONGO_URI", MONGO_DB),
        "ELASTICSEARCH_INDEX": ELASTICSEARCH_INDEX,
        "PRODAPI_URL": "http://localhost:5500",
        "MEDIA_PREFIX": "http://localhost:5500/prodapi/v1/assets",
        "PRODAPI_URL_PREFIX": "prodapi",
        "URL_PREFIX": "prodapi",
        "AUTH_SERVER_SHARED_SECRET": AUTH_SERVER_SHARED_SECRET,
    }
    if extra_config:
        test_config.update(extra_config)
    prodapi_app = get_prodapi_api(test_config)

    # put elastic mapping
    with prodapi_app.app_context():
        prodapi_app.data.elastic.init_index()

    return prodapi_app


def get_test_superdesk_app(extra_config=None):
    """
    Create and return configured test superdesk flask app.
    :param extra_config: extra settings
    :return: eve.flaskapp.Eve
    """
    test_config = {
        "MONGO_DBNAME": MONGO_DB,
        "MONGO_URI": get_mongo_uri("MONGO_URI", MONGO_DB),
        "ELASTICSEARCH_INDEX": ELASTICSEARCH_INDEX,
        "AUTH_SERVER_SHARED_SECRET": AUTH_SERVER_SHARED_SECRET,
    }
    if extra_config:
        test_config.update(extra_config)

    def context():
        pass

    context.app = None
    context.ctx = None
    context.client = None
    setup(context=context, config=test_config, app_factory=get_sd_app)

    return context.app


def teardown_app(app):
    """
    Drop test db and test app
    """
    clean_dbs(app)
    del app


@pytest.fixture(scope="function")
def superdesk_app(request):
    """
    Superdesk app.

    :return: superdesk app
    :rtype: superdesk.factory.app.SuperdeskEve
    """

    extra_config = getattr(request, "param", {})
    app = get_test_superdesk_app(extra_config)

    def test_app_teardown():
        """
        Drop test db and test app
        """
        teardown_app(app)

    request.addfinalizer(test_app_teardown)

    return app


@pytest.fixture(scope="function")
def prodapi_app(request):
    """
    Prod api app.

    :return: prod api app
    :rtype: eve.flaskapp.Eve
    """

    extra_config = getattr(request, "param", {})
    app = get_test_prodapi_app(extra_config)

    def test_app_teardown():
        """
        Drop test db and test app
        """
        teardown_app(app)

    request.addfinalizer(test_app_teardown)

    return app


@pytest.fixture(scope="module")
def prodapi_app_with_data(request):
    """
    Prod api app with prefilled collections and with disabled auth.
    ATTENTION: This is a resource-heavy fixture and it's designed to use with "module" scope.
    It's better to use it in readonly tests and not modify data of fixtured app.

    :return: prod api app
    :rtype: eve.flaskapp.Eve
    """

    extra_config = getattr(request, "param", {})
    extra_config["PRODAPI_AUTH_ENABLED"] = False
    app = get_test_prodapi_app(extra_config)

    # fill with data
    with app.app_context():
        p = Path(os.path.join(os.path.dirname(__file__), "tests/fixtures"))
        for fixture_file in [x for x in p.iterdir() if x.is_file()]:
            with fixture_file.open() as f:
                app.data.insert(resource=fixture_file.stem, docs=json.loads(f.read()))

    def test_app_teardown():
        """
        Drop test db and test app
        """
        teardown_app(app)

    request.addfinalizer(test_app_teardown)

    return app


@pytest.fixture(scope="module")
def prodapi_app_with_data_client(prodapi_app_with_data):
    """Test client for prod api with filled data"""

    client = prodapi_app_with_data.test_client()

    with prodapi_app_with_data.app_context():
        yield client


@pytest.fixture(scope="function")
def prodapi_client(prodapi_app):
    """Test client for prod api"""

    client = prodapi_app.test_client()

    with prodapi_app.app_context():
        yield client


@pytest.fixture(scope="function")
def superdesk_client(superdesk_app):
    """Test client for superdesk"""

    client = superdesk_app.test_client()

    with superdesk_app.app_context():
        yield client


@pytest.fixture(scope="function")
def auth_server_registered_clients(request, superdesk_app):
    """
    Registers clients for auth server.
    :return: dict with clients
    """
    clients_data = []

    with superdesk_app.app_context():
        for param in request.param:
            # register clients
            clients_data.append(
                {
                    "name": str(ObjectId()),  # just a random string
                    "client_id": str(ObjectId()),
                    "password": str(ObjectId()),  # just a random string
                    "scope": param,
                }
            )
            RegisterClient().run(**clients_data[-1])

    return clients_data


@pytest.fixture(scope="function")
def issued_tokens(request, superdesk_app, superdesk_client):
    tokens = []
    clients_data = []

    # register clients
    with superdesk_app.app_context():
        for param in request.param:
            clients_data.append(
                {
                    "name": str(ObjectId()),  # just a random string
                    "client_id": str(ObjectId()),
                    "password": str(ObjectId()),  # just a random string
                    "scope": param,
                }
            )
            RegisterClient().run(**clients_data[-1])

    # retrieve tokens
    with superdesk_app.test_request_context():
        for client_data in clients_data:
            resp = superdesk_client.post(
                url_for("auth_server.issue_token"),
                data={"grant_type": "client_credentials"},
                headers={"Authorization": _basic_auth_str(client_data["client_id"], client_data["password"])},
            )
            tokens.append(json.loads(resp.data.decode("utf-8")))

    teardown_app(superdesk_app)
    del superdesk_client

    return tokens



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

from datetime import datetime
from uuid import uuid4
from urllib.parse import urlparse
from flask import current_app as app
from contextlib import contextmanager

from superdesk.utils import SuperdeskBaseEnum
from .item import GUID_TAG, GUID_NEWSML, GUID_FIELD, ITEM_TYPE, CONTENT_TYPE


item_url = r'regex("[\w,.:_-]+")'

extra_response_fields = [GUID_FIELD, "headline", "firstcreated", "versioncreated", "firstpublished", "archived"]

aggregations = {
    "type": {"terms": {"field": "type"}},
    "desk": {"terms": {"field": "task.desk", "size": 50}},
    "category": {"terms": {"field": "anpa_category.name", "size": 50}},
    "source": {"terms": {"field": "source", "size": 50}},
    "urgency": {"terms": {"field": "urgency"}},
    "priority": {"terms": {"field": "priority"}},
    "legal": {"terms": {"field": "flags.marked_for_legal"}},
    "sms": {"terms": {"field": "flags.marked_for_sms"}},
    "genre": {"terms": {"field": "genre.name", "size": 50}},
}


def add_aggregation(aggregation_id, description):
    """
    Add aggregation to elasticsearch aggregations

    :param aggregation_id: string
    :param description: dict containing the aggregation schema
    """
    if not isinstance(aggregation_id, str):
        raise RuntimeError("Invalid aggregation identifier %s" % aggregation_id)
    if not isinstance(description, dict):
        raise RuntimeError("Invalid aggregation description for %s" % aggregation_id)
    aggregations[aggregation_id] = description


def remove_aggregation(aggregation_id):
    """
    Remove aggregation from elasticsearch aggregations

    :param aggregation_id: string
    """
    if aggregation_id in aggregations:
        del aggregations[aggregation_id]


@contextmanager
def aggregations_manager(aggregations):
    """
    Context manager used for temporarity applying a list of aggregations

    :param aggregations: iterable containing tuples of (aggregation_id, description)
    """
    for aggregation_id, description in aggregations:
        add_aggregation(aggregation_id, description)
    yield
    for aggregation_id, description in aggregations:
        remove_aggregation(aggregation_id)


def get_elastic_highlight_query(query_string):
    if query_string:
        field_settings = {"highlight_query": {"query_string": query_string}, "number_of_fragments": 0}

        elastic_highlight_query = {
            "require_field_match": False,
            "pre_tags": ['<span class="es-highlight">'],
            "post_tags": ["</span>"],
            "fields": {
                "body_html": field_settings,
                "body_footer": field_settings,
                "headline": field_settings,
                "slugline": field_settings,
                "abstract": field_settings,
            },
        }

        return elastic_highlight_query


def generate_guid(**hints):
    """Generate a GUID based on given hints

    param: hints: hints used for generating the guid
    """
    newsml_guid_format = "urn:newsml:%(domain)s:%(timestamp)s:%(identifier)s"
    tag_guid_format = "tag:%(domain)s:%(year)d:%(identifier)s"

    if not hints.get("id"):
        hints["id"] = str(uuid4())

    if app.config.get("GENERATE_SHORT_GUID", False):
        return hints["id"]

    t = datetime.today()

    if hints["type"].lower() == GUID_TAG:
        return tag_guid_format % {"domain": app.config["SERVER_DOMAIN"], "year": t.year, "identifier": hints["id"]}
    elif hints["type"].lower() == GUID_NEWSML:
        return newsml_guid_format % {
            "domain": app.config["SERVER_DOMAIN"],
            "timestamp": t.isoformat(),
            "identifier": hints["id"],
        }
    return None


def generate_tag(domain, id, prefix="tag"):
    """Generate tag for given domain and id.

    :param domain: domain string
    :param id: local id
    :param prefix
    """
    return "{}:{}:{}".format(prefix, domain, id)


def generate_tag_from_url(url, prefix="tag"):
    """Generate tag from given url.

    :param url
    :param prefix
    """
    parsed = urlparse(url)
    return generate_tag(parsed.netloc, parsed.path.lstrip("/").replace("/", ":"), prefix=prefix)


def is_normal_package(doc):
    """
    Returns True if the passed doc is a package. Otherwise, returns False.

    :return: True if it's a Package, False otherwise.
    """

    return doc[ITEM_TYPE] == CONTENT_TYPE.COMPOSITE


class ProductTypes(SuperdeskBaseEnum):
    API = "api"
    DIRECT = "direct"
    BOTH = "both"



from superdesk.emails import send_user_status_changed_email, send_activity_emails, send_email, send_translation_changed
from superdesk.tests import TestCase
from unittest.mock import patch


class SendEmailTestCase(TestCase):
    def test_send_email(self):
        with self.app.app_context():
            with self.app.mail.record_messages() as outbox:
                assert len(outbox) == 0
                send_user_status_changed_email(["test@sd.io"], "created")
                assert len(outbox) == 1
                assert outbox[0].subject == "Your Superdesk account is created"

    def test_send_email_multiline_subject(self):
        with self.app.app_context():
            with self.app.mail.record_messages() as outbox:
                send_email("foo\nbar", "admin@localhost", ["foo@example.com"], "text", "<p>html</p>")
                assert len(outbox) == 1
                assert outbox[0].subject == "foo"

    def test_send_activity_emails_error(self):
        recipients = ["foo", "bar"]
        activities = [
            {"message": "error", "data": {"foo": 1}},
            {"message": "error", "data": {"bar": 1}},
        ]
        with patch.object(send_email, "delay", return_value=None) as sent:
            with self.app.app_context():
                send_activity_emails(activities[0], recipients)
                self.assertEqual(1, sent.call_count)
                send_activity_emails(activities[0], recipients)
                self.assertEqual(1, sent.call_count)
                send_activity_emails(activities[1], recipients)
                self.assertEqual(2, sent.call_count)
                send_activity_emails(activities[1], recipients)
                self.assertEqual(2, sent.call_count)

    def test_send_translation_changed(self):
        item = {"_id": "test_id", "guid": "guid", "headline": "headline test"}
        with self.app.app_context():
            with self.app.mail.record_messages() as outbox:
                assert len(outbox) == 0
                send_translation_changed("admin", item, ["test@sd.io"])
                assert len(outbox) == 1
                assert outbox[0].subject == "The original item headline test has been changed"
                link = "http://localhost:9000/#/workspace?item=guid&action=edit"
                assert outbox[0].body.find(link) != -1
                assert outbox[0].html.find(link) != -1



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import io
import os
import hmac
import json
import flask
import unittest
import requests

from superdesk.publish import SUBSCRIBER_TYPES
from superdesk.publish.transmitters.http_push import HTTPPushService

from unittest import mock
from unittest.mock import Mock
from superdesk.errors import PublishHTTPPushServerError, PublishHTTPPushClientError


def get_fixture(fixture):
    filename = os.path.join(os.path.dirname(os.path.realpath(__file__)), "../", "fixtures", "%s.json" % fixture)
    with open(filename, "r") as file:
        return json.load(file)


class ItemNotFound(Exception):
    pass


class NotFoundResponse:
    status_code = 404


class CreatedResponse:
    status_code = 201


class CreatedResponseSession:
    def send(self, request):
        self._request = request
        return CreatedResponse()


class TestMedia(io.BytesIO):
    _id = "media-id"
    filename = "foo.txt"
    mimetype = "text/plain"


class HTTPPushServiceTestCase(unittest.TestCase):
    def setUp(self):
        if "HTTP_PUSH_RESOURCE_URL" not in os.environ:
            self.resource_url = ""
        else:
            self.resource_url = os.environ["HTTP_PUSH_RESOURCE_URL"]

        self.subscribers = [
            {
                "_id": "1",
                "name": "Test",
                "media_type": "media",
                "subscriber_type": SUBSCRIBER_TYPES.DIGITAL,
                "is_active": True,
                "sequence_num_settings": {"max": 10, "min": 1},
                "destinations": [
                    {
                        "name": "test",
                        "delivery_type": "http_push",
                        "format": "ninjs",
                        "config": {"resource_url": self.resource_url, "secret_token": "123456789"},
                    }
                ],
            }
        ]
        self.formatted_item1 = {
            "_id": "item1",
            "headline": "headline",
            "versioncreated": "2015-03-09T16:32:23",
            "version": 1,
        }
        self.formatted_item2 = {
            "_id": "item1",
            "headline": "headline2",
            "versioncreated": "2015-03-21T13:43:51",
            "version": 2,
        }
        self.item = {
            "item_id": "item1",
            "format": "ninjs",
            "item_version": 1,
            "published_seq_num": 1,
            "formatted_item": json.dumps(self.formatted_item1),
            "destination": {
                "name": "test",
                "delivery_type": "http_push",
                "format": "ninjs",
                "config": {"resource_url": self.resource_url, "secret_token": "123456789"},
            },
        }

        self.destination = self.item.get("destination", {})
        self.app = flask.Flask(__name__)

    def is_item_published(self, item_id):
        """Return True if the item was published, False otherwise.

        Raises Exception in case of server/communication error.
        """
        if not getattr(self, "resource_url", None):
            return

        response = requests.get(self.getItemURL(item_id))
        if response.status_code == requests.codes.not_found:  # @UndefinedVariable
            return False
        self.assertEqual(
            response.status_code, requests.codes.ok, "Error retrieving item from the content API"  # @UndefinedVariable
        )
        return True

    def getItemURL(self, item_id):
        """Returns the URL for item read

        @param item_id: the item identifier
        @return: string
        """
        return "%s/%s" % (self.resource_url, item_id)

    def test_get_assets_url(self):
        service = HTTPPushService()
        self.assertEqual(service._get_assets_url(self.destination), None)

    def test_get_resource_url(self):
        service = HTTPPushService()
        self.assertEqual(service._get_resource_url(self.destination), self.resource_url)

    def test_get_secret_token(self):
        service = HTTPPushService()
        self.assertEqual(service._get_secret_token(self.destination), "123456789")

    def test_get_headers(self):
        service = HTTPPushService()
        headers = service._get_headers("test payload", self.destination, {})
        self.assertEqual("sha1=8be62a607898504f87559cb52dc23f9ebee65a21", headers[service.hash_header])

    def test_publish_an_item(self):
        if not getattr(self, "resource_url", None):
            return

        service = HTTPPushService()

        service._transmit(self.item, self.subscribers)
        self.assertTrue(self.is_item_published(self.item["item_id"]))

        self.item["formatted_item"] = json.dumps(self.formatted_item2)
        service._transmit(self.item, self.subscribers)
        item = requests.get(self.getItemURL(self.item["item_id"])).json()
        self.assertEqual(item["headline"], "headline2")
        self.assertEqual(item["version"], 2)

    @mock.patch("superdesk.errors.notifiers")
    @mock.patch("requests.post")
    def test_client_publish_error_thrown(self, fake_post, fake_notifiers):
        with self.app.app_context():
            raise_http_exception = Mock(side_effect=PublishHTTPPushClientError.httpPushError(Exception("client 4xx")))

        fake_post.return_value = Mock(status_code=401, text="client 4xx", raise_for_status=raise_http_exception)

        # needed for bad exception handling classes
        fake_notifiers.return_value = []

        service = HTTPPushService()

        with self.assertRaises(PublishHTTPPushClientError):
            with self.app.app_context():
                service._push_item(self.destination, json.dumps(self.item))

    @mock.patch("superdesk.errors.notifiers")
    @mock.patch("requests.post")
    def test_server_publish_error_thrown(self, fake_post, fake_notifiers):
        with self.app.app_context():
            raise_http_exception = Mock(side_effect=PublishHTTPPushServerError.httpPushError(Exception("server 5xx")))

        fake_post.return_value = Mock(status_code=503, text="server 5xx", raise_for_status=raise_http_exception)

        # needed for bad exception handling classes
        fake_notifiers.return_value = []

        service = HTTPPushService()

        with self.assertRaises(PublishHTTPPushServerError):
            with self.app.app_context():
                service._push_item(self.destination, json.dumps(self.item))

    @mock.patch("superdesk.publish.transmitters.http_push.app")
    @mock.patch("superdesk.publish.transmitters.http_push.requests.Session.send", return_value=CreatedResponse)
    @mock.patch("requests.get", return_value=NotFoundResponse)
    def test_push_associated_assets(self, get_mock, send_mock, app_mock):
        app_mock.media.get.return_value = TestMedia(b"bin")

        dest = {"config": {"assets_url": "http://example.com"}}
        item = get_fixture("package")

        service = HTTPPushService()
        service._copy_published_media_files({}, dest)

        get_mock.assert_not_called()
        send_mock.assert_not_called()

        service._copy_published_media_files(item, dest)

        images = [
            # embedded original
            "2017020111028/9a836848c3c3387a151dbed96e83b7d50e6b0e71ca397e0b1dc0f4b2f4127acd.jpg",
            # main-0 original
            "20170201110216/d3ad29bafe0710c42b7cfc201939f266c6ca5c11a713625388decff4da87ba5b.jpg",
            # embedded thumbnail
            "2017020111028/a0502320d6d07dd921253171e971943adf791eb2b34dfe82da73c053a343a7c2.jpg",
        ]

        for media in images:
            get_mock.assert_any_call("http://example.com/%s" % media)

    @mock.patch("superdesk.publish.transmitters.http_push.app")
    @mock.patch("superdesk.publish.transmitters.http_push.requests.Session.send", return_value=CreatedResponse)
    @mock.patch("requests.get", return_value=NotFoundResponse)
    def test_push_attachments(self, get_mock, send_mock, app_mock):
        app_mock.media.get.return_value = TestMedia(b"bin")

        dest = {"config": {"assets_url": "http://example.com", "secret_token": "foo"}}
        item = {
            "type": "text",
            "attachments": [
                {"id": "foo", "media": "media-id", "mimetype": "text/plain"},
            ],
        }

        service = HTTPPushService()
        service._copy_published_media_files(item, dest)

        app_mock.media.get.assert_called_with("media-id", resource="attachments")
        get_mock.assert_called_with("http://example.com/media-id")
        send_mock.assert_called_once_with(mock.ANY)
        request = send_mock.call_args[0][0]
        self.assertEqual("http://example.com/", request.url)
        self.assertEqual("POST", request.method)
        self.assertIn(b"bin", request.body)
        self.assertIn(b"media-id", request.body)
        self.assertIn("x-superdesk-signature", request.headers)
        self.assertEqual(
            request.headers["x-superdesk-signature"], "sha1=%s" % hmac.new(b"foo", request.body, "sha1").hexdigest()
        )

    @mock.patch("superdesk.publish.transmitters.http_push.requests.Session.send", return_value=CreatedResponse)
    @mock.patch("requests.get", return_value=NotFoundResponse)
    def test_push_binaries(self, get_mock, send_mock):
        media = TestMedia(b"content")
        dest = {"config": {"assets_url": "http://example.com", "secret_token": "foo"}}
        service = HTTPPushService()
        service._transmit_media(media, dest)
        get_mock.assert_called_with("http://example.com/media-id")
        send_mock.assert_called_once_with(mock.ANY)
        request = send_mock.call_args[0][0]
        self.assertEqual("http://example.com/", request.url)
        self.assertIn(b"content", request.body)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import superdesk
from flask import request


class WorkspaceResource(superdesk.Resource):

    schema = {
        "name": {"type": "string", "unique_to_user": True},
        "widgets": {"type": "list"},
        "desk": superdesk.Resource.rel("desks"),
        "user": superdesk.Resource.rel("users"),
        "init_version": {"type": "integer"},
    }
    item_methods = ["GET", "PATCH", "DELETE"]


class WorkspaceService(superdesk.Service):
    def is_authorized(self, **kwargs):
        if kwargs.get("_id"):
            data = self.find_one(req=None, _id=kwargs.get("_id")) or request.get_json()
        else:
            data = request.get_json()
        # TODO(petr): use privileges to test who can save desk/role dashboard
        return "user" not in data or str(data["user"]) == str(kwargs["user_id"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.io.feed_parsers.newsml_2_0 import NewsMLTwoFeedParser
from superdesk.tests import TestCase
from superdesk.etree import etree
import os


class BaseAPTestCase(TestCase):

    vocab = [{"_id": "genre", "items": [{"name": "Current"}]}]

    def setUp(self):
        with self.app.app_context():
            self.app.data.insert("vocabularies", self.vocab)
        dirname = os.path.dirname(os.path.realpath(__file__))
        fixture = os.path.normpath(os.path.join(dirname, "../fixtures", self.filename))
        provider = {"name": "Test"}
        with open(fixture, "rb") as f:
            self.root_elt = etree.fromstring(f.read())
            self.items = NewsMLTwoFeedParser().parse(self.root_elt, provider)


class APTestCase(BaseAPTestCase):
    filename = "ap.xml"

    def test_can_parse(self):
        self.assertTrue(NewsMLTwoFeedParser().can_parse(self.root_elt))

    def test_content(self):
        item = self.items[0]
        self.assertEqual(item["version"], "2")
        self.assertEqual(item["headline"], "Nutrisystem beats 4Q earnings and revenue expectations")
        self.assertEqual(item["slugline"], "BC-US--Earns-Nutrisystem, 1st Ld-Writethru")
        self.assertEqual(
            item["body_html"],
            "<p>FORT WASHINGTON, Pa. (AP) _ Nutrisystem Inc. (NTRI) on Monday reported"
            " fourth-quarter net income of $10.9 million.</p>\n              \n<p>On a"
            " per-share basis, the Fort Washington, Pennsylvania-based company said it"
            " had profit of 36 cents. Earnings, adjusted for non-recurring costs, were"
            " 42 cents per share.</p>\n              \n<p>The results beat Wall Street"
            " expectations. The average estimate of four analysts surveyed by Zacks In"
            "vestment Research was for earnings of 41 cents per share.</p>\n          "
            "    \n<p>The weight-loss company posted revenue of $131.2 million in the "
            "period, also exceeding Street forecasts. Four analysts surveyed by Zacks "
            "expected $129.1 million.</p>\n              \n<p>For the year, the compan"
            "y reported profit of $57.9 million, or $1.90 per share. Revenue was repor"
            "ted as $697 million.</p>\n              \n<p>For the current quarter endi"
            "ng in April, Nutrisystem said it expects revenue in the range of $204 mil"
            "lion to $209 million.</p>\n              \n<p>The company expects full-ye"
            "ar earnings to be $1.99 to $2.09 per share, with revenue ranging from $68"
            "5 million to $705 million.</p>\n              \n<p>Nutrisystem shares hav"
            "e decreased 24 percent since the beginning of the year. In the final minu"
            "tes of trading on Monday, shares hit $39.90, a climb of almost 5 percent "
            "in the last 12 months.</p>\n              \n<p>_____</p>\n              "
            "\n<p>This story was generated by Automated Insights (http://automatedinsig"
            "hts.com/ap) using data from Zacks Investment Research. Access a Zacks sto"
            "ck report on NTRI at https://www.zacks.com/ap/NTRI</p>\n            ",
        )
        self.assertEqual(item["guid"], "tag:ap.org,2011:5e565888b587889c5c8bd62b6164118c-text:2")
        self.assertEqual(item["firstcreated"].isoformat(), "2018-02-26T21:20:36+00:00")



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


import logging
import superdesk

from .validators import ValidatorsService, ValidatorsResource

logger = logging.getLogger(__name__)


def init_app(app):
    endpoint_name = "validators"
    service = ValidatorsService(endpoint_name, backend=superdesk.get_backend())
    ValidatorsResource(endpoint_name, app=app, service=service)



import bson
import logging
import superdesk

from superdesk.utc import utcnow
from superdesk.metadata.item import ITEM_STATE, PUBLISH_STATES


logger = logging.getLogger(__name__)


def track_usage(media_item, stored_item, item_obj, item_name, original):
    if not media_item:
        return

    try:
        orig_id = original["associations"][item_name]["_id"]
    except (AttributeError, TypeError, KeyError):
        orig_id = None

    if item_obj["_id"] != orig_id:
        _update_usage(media_item)
        stored_item["used"] = True


def _update_usage(item):
    updates = {
        "used": True,
        "used_count": item.get("used_count", 0) + 1,
        "used_updated": utcnow(),
    }

    superdesk.get_resource_service("archive").system_update(item["_id"], updates, item)

    # update published item state as well
    if item.get(ITEM_STATE) in PUBLISH_STATES:
        published = superdesk.get_resource_service("published").get_last_published_version(item["_id"])
        if published:
            superdesk.get_resource_service("published").system_update(
                bson.ObjectId(published["_id"]), updates, published
            )
        else:
            logger.warning("published item not found for item %s", item["_id"])

    item.update(updates)


def update_refs(updates, original):
    """Update refs stored on item based on its associations.

    We can't use associations for queries due to unknown keys in dict,
    so storing basic metadata again as list in `refs`.
    """
    if "associations" not in updates:
        return
    refs = []
    assoc = original["associations"].copy() if original.get("associations") else {}
    assoc.update(updates["associations"] or {})
    for key, val in assoc.items():
        if not val:
            continue
        if val.get("_id") and not val.get("guid"):
            # for related items we only store the _id, fetch other metadata
            item = superdesk.get_resource_service("archive").find_one(req=None, _id=val["_id"]) or {}
        else:
            item = {}
        refs.append(
            {
                "key": key,
                "_id": val.get("_id"),
                "uri": val.get("uri") or item.get("uri"),
                "guid": val.get("guid") or item.get("guid"),
                "type": val.get("type") or item.get("type"),
            }
        )
    updates["refs"] = refs



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest

from superdesk.io.mediatopics import MediaTopicsIndex
from superdesk.io.iptc import subject_codes


class MediaTopicsTestCase(unittest.TestCase):
    def setUp(self):
        self.topics = MediaTopicsIndex()

    def test_all_subjects_map_to_a_media_topic(self):
        for (k, v) in sorted(subject_codes.items()):
            topic = self.topics.get_media_topic(k)
            self.assertIsNotNone(topic)
            # print(k, v, '\t---->\t',topic, self.topics.get_media_topic_item(topic).get('prefLabel').get('en-GB'))

    def test_get_items(self):
        items = self.topics.get_items()
        for i in items:
            subject = self.topics.get_subject_code(i.get("qcode"))
            self.assertIsNotNone(subject)
            # print(i.get('qcode'), i.get('name'), '\t---->\t', subject, subject_codes.get(subject))

    def test_get_media_topics(self):
        items = self.topics.get_media_topics()
        for (i, v) in items.items():
            subject = self.topics.get_subject_code(i)
            self.assertIsNotNone(subject)
            # print(i, v.get('prefLabel').get('en-GB'), '\t---->\t', subject, subject_codes.get(subject))

    def test_media_topic_to_subject(self):
        subject = self.topics.get_subject_code("medtop:20000070")
        self.assertEqual(subject, "16004000")

    def test_subject_to_media_topic(self):
        topic = self.topics.get_media_topic("16004000")
        self.assertEqual(topic, "medtop:20000070")

    def test_get_media_topic_item(self):
        topic = self.topics.get_media_topic_item("medtop:20000070")
        self.assertEqual(topic["type"], ["http://www.w3.org/2004/02/skos/core#Concept"])



from unittest.mock import Mock, call
from datetime import timedelta

from superdesk.tests import TestCase
from superdesk.utc import utcnow
from superdesk.utc import query_datetime

from superdesk.commands.remove_exported_files import RemoveExportedFiles


class MockMediaFS:
    """Simple Mocked Media FS"""

    def __init__(self, files):
        self.files = files
        self.delete = Mock()
        self.delete.side_effect = self._delete
        self.find = Mock()
        self.find.side_effect = self._find

    def _delete(self, _id):
        del self.files[_id]

    def _find(self, folder=None, upload_date=None):
        files = []
        for file in self.files.values():
            if (upload_date is not None and not query_datetime(file["upload_date"], upload_date)) or not file[
                "filename"
            ].startswith(folder):
                continue
            files.append(file)
        return files


class RemoveExportedFilesTest(TestCase):
    def setUp(self):
        # Create a backup of the original media reference so we can set it back up again later
        self.original_media = self.app.media
        self.now = utcnow()
        self.app.media = MockMediaFS(
            {
                "a1": {"_id": "a1", "filename": "aa", "upload_date": self.now},
                "b2": {"_id": "b2", "filename": "bb", "upload_date": self.now - timedelta(hours=20)},
                "c3": {"_id": "c3", "filename": "cc", "upload_date": self.now - timedelta(hours=25)},
                "d4": {"_id": "d4", "filename": "dd", "upload_date": self.now - timedelta(hours=36)},
                "e5": {"_id": "e5", "filename": "temp/ee", "upload_date": self.now},
                "f6": {"_id": "f6", "filename": "temp/ff", "upload_date": self.now - timedelta(hours=20)},
                "g7": {"_id": "g7", "filename": "temp/gg", "upload_date": self.now - timedelta(hours=25)},
                "h8": {"_id": "h8", "filename": "temp/hh", "upload_date": self.now - timedelta(hours=36)},
            }
        )
        self.can_expire = ["g7", "h8"]

        self.command = RemoveExportedFiles()

    def tearDown(self):
        """Ensure to restore the original media reference"""
        self.app.media = self.original_media

    def test_get_file_ids(self):
        expire_at = utcnow() - timedelta(hours=24)
        for file_id in self.command._get_file_ids(expire_at):
            self.assertIn(file_id, self.can_expire)

        kwargs = {"folder": "temp", "upload_date": {"$lte": expire_at}}
        self.app.media.find.assert_called_once_with(**kwargs)

    def test_remove_exported_files(self):
        expire_at = utcnow() - timedelta(hours=24)
        self.command._remove_exported_files(expire_at)

        for file_id in self.app.media.files:
            self.assertNotIn(file_id, self.can_expire)

        self.app.media.delete.assert_has_calls([call("g7"), call("h8")], any_order=True)
        self.assertEqual(self.app.media.delete.call_count, 2)

    def test_run(self):
        self.command.run()
        for file_id in self.app.media.files:
            self.assertNotIn(file_id, self.can_expire)

        self.app.media.delete.assert_has_calls([call("g7"), call("h8")], any_order=True)
        self.assertEqual(self.app.media.delete.call_count, 2)



from flask import current_app as app
from superdesk import get_resource_service


def add_missing_keywords(sender, item, **kwargs):
    if app.config.get("KEYWORDS_ADD_MISSING_ON_PUBLISH") and item.get("keywords"):
        get_resource_service("vocabularies").add_missing_keywords(item["keywords"], item.get("language"))



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime

import superdesk
from superdesk.tests import TestCase


class ArchiveGetItemsChainTestCase(TestCase):
    archive = [
        {
            "_id": "original",
            "guid": "original",
            "headline": "Original EN",
            "translations": ["original-translation-fr", "original-translation-nl"],
            "rewritten_by": "update-1",
            "language": "en",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "original-translation-fr",
            "guid": "original-translation-fr",
            "headline": "Original Translation FR",
            "translated_from": "original",
            "language": "fr",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "original-translation-nl",
            "guid": "original-translation-nl",
            "headline": "Original Translation NL",
            "translated_from": "original",
            "language": "nl",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "update-1",
            "guid": "update-1",
            "headline": "Update 1 EN",
            "translations": [],
            "rewritten_by": "update-2",
            "rewrite_of": "original",
            "language": "en",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "update-2",
            "guid": "update-2",
            "headline": "Update 2 EN",
            "translations": [
                "update-2-translation-fr",
            ],
            "rewrite_of": "update-1",
            "language": "en",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "update-2-translation-fr",
            "guid": "update-2-translation-fr",
            "headline": "Update 2 Translation FR",
            "translated_from": "update-2",
            "translations": [
                "update-2-translation-fr-nl",
            ],
            "language": "fr",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "update-2-translation-fr-nl",
            "guid": "update-2-translation-fr-nl",
            "headline": "Update 2 Translation FR NL",
            "translated_from": "update-2-translation-fr",
            "language": "nl",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "update-2-translation-fr-nl-update",
            "guid": "update-2-translation-fr-nl-update",
            "headline": "Update 2 Translation FR NL update",
            "rewrite_of": "update-2-translation-fr-nl",
            "language": "nl",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
        {
            "_id": "update-2-translation-fr-nl-update-2",
            "guid": "update-2-translation-fr-nl-update-2",
            "headline": "Update 2 Translation FR NL update",
            "rewrite_of": "update-2-translation-fr-nl-update",
            "language": "nl",
            "type": "text",
            "version": 1,
            "profile": "text",
            "pubstatus": "usable",
            "format": "HTML",
            "firstcreated": datetime.datetime(2019, 4, 3, 12, 41, 53),
            "versioncreated": datetime.datetime(2019, 4, 3, 12, 45, 14),
            "original_creator": "5d385f31fe985ec67a0ca583",
            "state": "published",
            "source": "Belga",
            "version_creator": "5d385f31fe985ec67a0ca583",
            "slugline": "skoda scala",
            "byline": "BELGA",
        },
    ]

    def setUp(self):
        self.app.data.insert("archive", self.archive)

    def test_get_items_chain(self):
        archive_service = superdesk.get_resource_service("archive")

        original = self.archive[0]
        items = archive_service.get_items_chain(original)
        self.assertListEqual(
            [i["_id"] for i in items], ["original", "original-translation-fr", "original-translation-nl"]
        )

        original_translation_fr = self.archive[1]
        items = archive_service.get_items_chain(original_translation_fr)
        self.assertListEqual(
            [i["_id"] for i in items], ["original", "original-translation-fr", "original-translation-nl"]
        )

        original_translation_nl = self.archive[2]
        items = archive_service.get_items_chain(original_translation_nl)
        self.assertListEqual(
            [i["_id"] for i in items], ["original", "original-translation-fr", "original-translation-nl"]
        )

        update_1 = self.archive[3]
        items = archive_service.get_items_chain(update_1)
        self.assertListEqual(
            [i["_id"] for i in items], ["original", "original-translation-fr", "original-translation-nl", "update-1"]
        )

        update_2 = self.archive[4]
        items = archive_service.get_items_chain(update_2)
        self.assertListEqual(
            [i["_id"] for i in items],
            [
                "original",
                "original-translation-fr",
                "original-translation-nl",
                "update-1",
                "update-2",
                "update-2-translation-fr",
                "update-2-translation-fr-nl",
            ],
        )

        update_2_translation_fr = self.archive[5]
        items = archive_service.get_items_chain(update_2_translation_fr)
        self.assertListEqual(
            [i["_id"] for i in items],
            [
                "original",
                "original-translation-fr",
                "original-translation-nl",
                "update-1",
                "update-2",
                "update-2-translation-fr",
                "update-2-translation-fr-nl",
            ],
        )

        update_2_translation_fr_nl = self.archive[6]
        items = archive_service.get_items_chain(update_2_translation_fr_nl)
        self.assertListEqual(
            [i["_id"] for i in items],
            [
                "original",
                "original-translation-fr",
                "original-translation-nl",
                "update-1",
                "update-2",
                "update-2-translation-fr",
                "update-2-translation-fr-nl",
            ],
        )

        update_2_translation_fr_nl_update = self.archive[7]
        items = archive_service.get_items_chain(update_2_translation_fr_nl_update)
        self.assertListEqual(
            [i["_id"] for i in items],
            [
                "original",
                "original-translation-fr",
                "original-translation-nl",
                "update-1",
                "update-2",
                "update-2-translation-fr",
                "update-2-translation-fr-nl",
                "update-2-translation-fr-nl-update",
            ],
        )

        update_2_translation_fr_nl_update_2 = self.archive[8]
        items = archive_service.get_items_chain(update_2_translation_fr_nl_update_2)
        self.assertListEqual(
            [i["_id"] for i in items],
            [
                "original",
                "original-translation-fr",
                "original-translation-nl",
                "update-1",
                "update-2",
                "update-2-translation-fr",
                "update-2-translation-fr-nl",
                "update-2-translation-fr-nl-update",
                "update-2-translation-fr-nl-update-2",
            ],
        )



# -*- coding: utf-8; -*-
# This file is part of Superdesk.
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license
#
# Author  : mugur
# Creation: 2017-11-10 17:06

from superdesk.commands.data_updates import BaseDataUpdate


class DataUpdate(BaseDataUpdate):

    resource = "archive"

    def forwards(self, mongodb_collection, mongodb_database):
        print("Please rebuild elastic index after upgrade")

    def backwards(self, mongodb_collection, mongodb_database):
        pass



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import os
from json import loads
from .subjectcodes import get_parent_subjectcode
import logging

logger = logging.getLogger(__name__)


class MediaTopicsIndex:
    def __init__(self):
        # A map for mapping from an IPTC subject code to a Media Topic
        self.subject_media_topic_map = dict()
        # A map for mapping from a Media Topic code to a subject code.
        self.media_topic_to_subject_map = dict()
        # A map that contains all the media topics.
        self.media_topic_map = dict()

        dirname = os.path.dirname(os.path.realpath(__file__))

        # The media topics json file is available at http://cv.iptc.org/newscodes/mediatopic?lang=x-all
        # there are also language specific files that could be used.
        media_topic_codes = os.path.join(dirname, "data", "cptall-x-all.json")
        mediatopics = self._load_codes(media_topic_codes)
        self.generate_mediatopic_to_subject_map(mediatopics.get("conceptSet"))

    def _load_codes(self, filename):
        with open(filename, "r", encoding="UTF-8") as f:
            return loads(f.read())

    def clear_maps(self):
        """Clear the maps to enable them to be regenerated

        :return:
        """
        self.subject_media_topic_map = dict()
        self.media_topic_to_subject_map = dict()

    def generate_mediatopic_to_subject_map(self, mediatopics):
        """Scan the the media topics and create lookup maps for mapping subject codes to media topics and
        media topics to subjects

        :param mediatopics:
        :return:
        """
        working_map = dict()

        def set_map_entry(topic, match_type):
            match_list = topic.get(match_type, [])
            if len(match_list):
                for s in match_list:
                    if s.startswith("http://cv.iptc.org/newscodes/subjectcode/"):
                        subject_qcode = s.split("/")[-1]
                        if subject_qcode not in working_map:
                            working_map[subject_qcode] = {match_type: [topic.get("qcode")]}
                        else:
                            if match_type in working_map[subject_qcode]:
                                working_map[subject_qcode][match_type].append(topic.get("qcode"))
                            else:
                                working_map[subject_qcode][match_type] = [topic.get("qcode")]
                        if topic.get("qcode") not in self.media_topic_to_subject_map:
                            self.media_topic_to_subject_map[t.get("qcode")] = subject_qcode

        # For each media topic build match lists for each of the match types
        for t in mediatopics:
            # Save each Media Topic
            self.media_topic_map[t.get("qcode")] = t

            # Extract each match list
            set_map_entry(t, "exactMatch")
            set_map_entry(t, "closeMatch")
            set_map_entry(t, "broadMatch")

        # Construct a list in order of how close the match is, in some cases there may be for example multiple exact
        # matches, in this case the first one is chosen
        for (k, v) in working_map.items():
            matches = v.get("exactMatch", []) if "exactMatch" in v else []
            matches = matches + (v.get("closeMatch", []) if "closeMatch" in v else [])
            matches = matches + (v.get("broadMatch", []) if "broadMatch" in v else [])
            self.subject_media_topic_map[k] = matches[0] if len(matches) else None

    def get_subject_code(self, qcode):
        """Given a Media Topic qcode return the corresponding subject code, if a direct match cannot be found we try the
        parents

        :param qcode:
        :return:
        """
        current_topic = qcode
        subject = self.media_topic_to_subject_map.get(qcode)
        if subject:
            return subject
        while not subject:
            parent_topic = self._get_parent(self.get_media_topic_item(current_topic))
            if not parent_topic:
                return None
            subject = self.media_topic_to_subject_map.get(parent_topic)
            if subject:
                return subject
            current_topic = parent_topic

    def get_media_topic(self, subject):
        """Given a subject code return the closest matching media topic qcode

        :param subject:
        :return:
        """
        current_subject = subject
        qcode = self.subject_media_topic_map.get(current_subject)
        if qcode:
            return qcode
        while not qcode:
            parent_subject = get_parent_subjectcode(current_subject)
            if not parent_subject:
                return None
            qcode = self.subject_media_topic_map.get(parent_subject)
            if qcode:
                return qcode
            current_subject = parent_subject

    def get_media_topic_item(self, qcode):
        """Given the qcode for a media topic return the entire structure

        :param qcode:
        :return:
        """
        return self.media_topic_map.get(qcode, None)

    def get_media_topics(self):
        return self.media_topic_map

    def _get_pref_label(self, topic, language):
        """Extract a label for the media topic in the requested language

        :param topic:
        :param language:
        :return:
        """
        label = topic.get("prefLabel").get(language)
        if not label:
            # Not all languages are fully populated, but English seems to be.
            label = topic.get("prefLabel").get("en-GB")
            if not label:
                # Just return the first available
                key = list(topic.get("prefLabel").keys())[0]
                return topic.get("prefLabel").get(key)
        return label

    def _get_parent(self, topic):
        return (
            "medtop:" + topic.get("broader")[0].split("/")[-1]
            if len(topic.get("broader", []))
            and topic.get("broader")[0].startswith("http://cv.iptc.org/newscodes/mediatopic/")
            else None
        )

    def get_items(self, language="en-GB"):
        """Get list of all subjects.

        Each topic is a dict with `qcode`, `name` and `parent` and list of 'children' keys.
        """
        items = []
        for (_k, code) in self.media_topic_map.items():
            if not code.get("retired"):
                items.append(
                    {
                        "qcode": code.get("qcode"),
                        "name": self._get_pref_label(code, language),
                        "parent": self._get_parent(code),
                        "children": code.get("narrower"),
                    }
                )
        return sorted(items, key=lambda k: k["name"])


def init_app(app):
    app.mediatopics = MediaTopicsIndex()



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013 - 2018 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


# codes extracted from IPTC IIM v4.2 specifications


class TAG:
    RECORD_VERSION = "Record Version"
    OBJECT_TYPE_REFERENCE = "Object Type Reference"
    OBJECT_ATTRIBUTE_REFERENCE = "Object Attribute Reference"
    OBJECT_NAME = "Object Name"
    EDIT_STATUS = "Edit Status"
    EDITORIAL_UPDATE = "Editorial Update"
    URGENCY = "Urgency"
    SUBJECT_REFERENCE = "Subject Reference"
    CATEGORY = "Category"
    SUPPLEMENTAL_CATEGORY = "Supplemental Category"
    FIXTURE_IDENTIFIER = "Fixture Identifier"
    KEYWORDS = "Keywords"
    CONTENT_LOCATION_CODE = "Content Location Code"
    CONTENT_LOCATION_NAME = "Content Location Name"
    RELEASE_DATE = "Release Date"
    RELEASE_TIME = "Release Time"
    EXPIRATION_DATE = "Expiration Date"
    EXPIRATION_TIME = "Expiration Time"
    SPECIAL_INSTRUCTIONS = "Special Instructions"
    ACTION_ADVISED = "Action Advised"
    REFERENCE_SERVICE = "Reference Service"
    REFERENCE_DATE = "Reference Date"
    REFERENCE_NUMBER = "Reference Number"
    DATE_CREATED = "Date Created"
    TIME_CREATED = "Time Created"
    DIGITAL_CREATION_DATE = "Digital Creation Date"
    DIGITAL_CREATION_TIME = "Digital Creation Time"
    ORIGINATING_PROGRAM = "Originating Program"
    PROGRAM_VERSION = "Program Version"
    OBJECT_CYCLE = "Object Cycle"
    BY_LINE = "By-line"
    BY_LINE_TITLE = "By-line Title"
    CITY = "City"
    SUBLOCATION = "Sublocation"
    PROVINCE_STATE = "Province/State"
    COUNTRY_PRIMARY_LOCATION_CODE = "Country/Primary Location Code"
    COUNTRY_PRIMARY_LOCATION_NAME = "Country/Primary Location Name"
    ORIGINAL_TRANSMISSION_REFERENCE = "Original Transmission Reference"
    HEADLINE = "Headline"
    CREDIT = "Credit"
    SOURCE = "Source"
    COPYRIGHT_NOTICE = "Copyright Notice"
    CONTACT = "Contact"
    CAPTION_ABSTRACT = "Caption/Abstract"
    WRITER_EDITOR = "Writer/Editor"
    RASTERIZED_CAPTION = "Rasterized Caption"
    IMAGE_TYPE = "Image Type"
    IMAGE_ORIENTATION = "Image Orientation"
    LANGUAGE_IDENTIFIER = "Language Identifier"
    AUDIO_TYPE = "Audio Type"
    AUDIO_SAMPLING_RATE = "Audio Sampling Rate"
    AUDIO_SAMPLING_RESOLUTION = "Audio Sampling Resolution"
    AUDIO_DURATION = "Audio Duration"
    AUDIO_OUTCUE = "Audio Outcue"
    OBJECTDATA_PREVIEW_FILE_FORMAT = "ObjectData Preview File Format"
    OBJECTDATA_PREVIEW_FILE_FORMAT_VERSION = "ObjectData Preview File Format Version"
    OBJECTDATA_PREVIEW_DATA = "ObjectData Preview Data"


iim_codes = {
    (2, 0): TAG.RECORD_VERSION,
    (2, 3): TAG.OBJECT_TYPE_REFERENCE,
    (2, 4): TAG.OBJECT_ATTRIBUTE_REFERENCE,
    (2, 5): TAG.OBJECT_NAME,
    (2, 7): TAG.EDIT_STATUS,
    (2, 8): TAG.EDITORIAL_UPDATE,
    (2, 10): TAG.URGENCY,
    (2, 12): TAG.SUBJECT_REFERENCE,
    (2, 15): TAG.CATEGORY,
    (2, 20): TAG.SUPPLEMENTAL_CATEGORY,
    (2, 22): TAG.FIXTURE_IDENTIFIER,
    (2, 25): TAG.KEYWORDS,
    (2, 26): TAG.CONTENT_LOCATION_CODE,
    (2, 27): TAG.CONTENT_LOCATION_NAME,
    (2, 30): TAG.RELEASE_DATE,
    (2, 35): TAG.RELEASE_TIME,
    (2, 37): TAG.EXPIRATION_DATE,
    (2, 38): TAG.EXPIRATION_TIME,
    (2, 40): TAG.SPECIAL_INSTRUCTIONS,
    (2, 42): TAG.ACTION_ADVISED,
    (2, 45): TAG.REFERENCE_SERVICE,
    (2, 47): TAG.REFERENCE_DATE,
    (2, 50): TAG.REFERENCE_NUMBER,
    (2, 55): TAG.DATE_CREATED,
    (2, 60): TAG.TIME_CREATED,
    (2, 62): TAG.DIGITAL_CREATION_DATE,
    (2, 63): TAG.DIGITAL_CREATION_TIME,
    (2, 65): TAG.ORIGINATING_PROGRAM,
    (2, 70): TAG.PROGRAM_VERSION,
    (2, 75): TAG.OBJECT_CYCLE,
    (2, 80): TAG.BY_LINE,
    (2, 85): TAG.BY_LINE_TITLE,
    (2, 90): TAG.CITY,
    (2, 92): TAG.SUBLOCATION,
    (2, 95): TAG.PROVINCE_STATE,
    (2, 100): TAG.COUNTRY_PRIMARY_LOCATION_CODE,
    (2, 101): TAG.COUNTRY_PRIMARY_LOCATION_NAME,
    (2, 103): TAG.ORIGINAL_TRANSMISSION_REFERENCE,
    (2, 105): TAG.HEADLINE,
    (2, 110): TAG.CREDIT,
    (2, 115): TAG.SOURCE,
    (2, 116): TAG.COPYRIGHT_NOTICE,
    (2, 118): TAG.CONTACT,
    (2, 120): TAG.CAPTION_ABSTRACT,
    (2, 122): TAG.WRITER_EDITOR,
    (2, 125): TAG.RASTERIZED_CAPTION,
    (2, 130): TAG.IMAGE_TYPE,
    (2, 131): TAG.IMAGE_ORIENTATION,
    (2, 135): TAG.LANGUAGE_IDENTIFIER,
    (2, 150): TAG.AUDIO_TYPE,
    (2, 151): TAG.AUDIO_SAMPLING_RATE,
    (2, 152): TAG.AUDIO_SAMPLING_RESOLUTION,
    (2, 153): TAG.AUDIO_DURATION,
    (2, 154): TAG.AUDIO_OUTCUE,
    (2, 200): TAG.OBJECTDATA_PREVIEW_FILE_FORMAT,
    (2, 201): TAG.OBJECTDATA_PREVIEW_FILE_FORMAT_VERSION,
    (2, 202): TAG.OBJECTDATA_PREVIEW_DATA,
}



from superdesk import etree as sd_etree
from superdesk.tests import TestCase
from lxml import etree, html
from textwrap import dedent


class ParseHtmlTestCase(TestCase):
    def test_encode_carriage_return(self):
        text = "This is first line.\r\nThis is second line.\r\n"
        parsed = sd_etree.parse_html(text)
        self.assertEqual(text.replace("\r", "&#13;"), sd_etree.to_string(parsed))

        text = "<pre>This is first line.\r\nThis is second line.\r\n</pre>"
        parsed = sd_etree.parse_html(text, content="html")
        self.assertEqual(text.replace("\r", "&#13;"), sd_etree.to_string(parsed))

    def test_void_elements_fix(self):
        html_raw = "<p>this is a test with empty <h3/> non-void <em/> elements and a void <br/> one</p>"
        expected = "<p>this is a test with empty <h3></h3> non-void <em></em> elements and a void <br/> one</p>"
        parsed = sd_etree.parse_html(html_raw)
        sd_etree.fix_html_void_elements(parsed)
        self.assertEqual(sd_etree.to_string(parsed), expected)

    def test_clean_html(self):
        html_raw = dedent(
            """\
        <div>
           <header>this header must be removed</header>
           <p class="class_to_remove">
               <unknown_tag>bla
                   <strong>keep it strong</strong>
               </unknown_tag>
               <script>no script here !</script>
           </p>
        </div>
        """
        )
        elem = html.fromstring(html_raw)
        elem = sd_etree.clean_html(elem)
        expected = dedent(
            """\
        <div>
           this header must be removed
           <p>
               bla
                   <strong>keep it strong</strong>


           </p>
        </div>
        """
        )
        self.assertEqual(dedent(etree.tostring(elem, encoding="unicode")), expected)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

"""Superdesk Publish

This module handles transmission of items to respective destinations.
Items must be inserted to publish queue in order to get transmitted.
"""

import logging
from typing import NamedTuple

from superdesk.celery_app import celery
from superdesk.publish.publish_content import PublishContent
from superdesk import get_backend

logger = logging.getLogger(__name__)

registered_transmitters = {}
transmitter_errors = {}
registered_transmitters_list = []


class SubscriberTypes(NamedTuple):
    DIGITAL: str
    WIRE: str
    ALL: str


SUBSCRIBER_TYPES: SubscriberTypes = SubscriberTypes("digital", "wire", "all")


class SubscriberMediaTypes(NamedTuple):
    MEDIA: str
    NONMEDIA: str
    BOTH: str


SUBSCRIBER_MEDIA_TYPES: SubscriberMediaTypes = SubscriberMediaTypes("media", "non-media", "both")


def register_transmitter(transmitter_type, transmitter, errors):
    registered_transmitters[transmitter_type] = transmitter
    transmitter_errors[transmitter_type] = dict(errors)
    registered_transmitters_list.append(
        {
            "type": transmitter_type,
            "name": transmitter.NAME or transmitter_type,
            "config": getattr(transmitter, "CONFIG", None),
        }
    )


@celery.task(soft_time_limit=1800, expires=10)
def transmit():
    """Transmit items from ``publish_queue`` collection."""
    PublishContent().run()


# must be imported for registration
from superdesk.publish.subscribers import SubscribersResource, SubscribersService  # NOQA
from superdesk.publish.publish_queue import PublishQueueResource, PublishQueueService  # NOQA
from superdesk.publish.subscriber_token import SubscriberTokenResource, SubscriberTokenService  # NOQA


def init_app(app):
    # XXX: we need to do imports for transmitters and formatters here
    #      so classes creation is done after PublishService is set
    #      this is a temporary workaround until a proper plugin system
    #      is implemented in Superdesk
    import superdesk.publish.transmitters  # NOQA
    import superdesk.publish.formatters  # NOQA

    endpoint_name = "subscribers"
    service = SubscribersService(endpoint_name, backend=get_backend())
    SubscribersResource(endpoint_name, app=app, service=service)

    endpoint_name = "publish_queue"
    service = PublishQueueService(endpoint_name, backend=get_backend())
    PublishQueueResource(endpoint_name, app=app, service=service)

    superdesk.register_resource("subscriber_token", SubscriberTokenResource, SubscriberTokenService)

    app.client_config.update(
        {
            "transmitter_types": registered_transmitters_list,
        }
    )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest
from .volume_cubic_inches_to_metric import cubic_inches_to_metric


class VolumeTestCase(unittest.TestCase):
    def test(self):
        text = (
            "Total volume is 100.50 cubic inches for this land. "
            "Total volume is 15.7 cubic in for this land. "
            "Total volume is 1 Cubic Inch for this land. "
            "Total volume is 1-16 cu-in for this land. "
            "Total volume is 1-16 cb. in for this land. "
            "Total volume is 16.7-Cubic-in for this land. "
            "Total volume is 16,500-cu. in. for this land. "
        )
        item = {"body_html": text}
        res, diff = cubic_inches_to_metric(item)
        self.assertEqual(diff["100.50 cubic inches"], "100.50 cubic inches (1,647 cubic centimeter)")
        self.assertEqual(diff["15.7 cubic in"], "15.7 cubic in (257.3 cubic centimeter)")
        self.assertEqual(diff["1 Cubic Inch"], "1 Cubic Inch (16 cubic centimeter)")
        self.assertEqual(diff["1-16 cu-in"], "1-16 cu-in (16-262 cubic centimeter)")
        self.assertEqual(diff["1-16 cb. in"], "1-16 cb. in (16-262 cubic centimeter)")
        self.assertEqual(diff["16.7-Cubic-in"], "16.7-Cubic-in (273.7 cubic centimeter)")
        self.assertEqual(diff["16,500-cu. in"], "16,500-cu. in (0.3 cubic meter)")
        self.assertEqual(res["body_html"], item["body_html"])



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import unittest

from superdesk.macros.extract_html import extract_html_macro


class ExtractHtmlTestCase(unittest.TestCase):
    def test_extract_html_macro(self):
        item = {"body_html": 'test <a href="www.sourcefabric.org">Sourcefabric</a> <br> <span>test <span/>'}
        extract_html_macro(item)
        self.assertEqual(
            item.get("body_html"), '<p>test <a href="www.sourcefabric.org">Sourcefabric</a> </p><p>test </p>'
        )



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license


from superdesk.metadata.item import ITEM_STATE

states = []
actions = []
allowed_workflow_states = []

__all__ = (
    "workflow_state",
    "get_workflow_states",
    "allowed_workflow_states",
    "workflow_action",
    "get_workflow_actions",
    "is_workflow_state_transition_valid",
    "set_default_state",
)


def workflow_action(name, include_states=None, exclude_states=None, privileges=None):
    """Register new workflow action.

    :param name: unique name of action
    :param include_states: include action if item state is in include_states
    :param exclude_states: exclude action if item state is in exclude_statese
    :param privileges: list of privileges required for this action
    """
    if include_states is None:
        include_states = []
    if exclude_states is None:
        exclude_states = []
    if privileges is None:
        privileges = []
    actions.append(
        {"name": name, "exclude_states": exclude_states, "include_states": include_states, "privileges": privileges}
    )


def get_workflow_actions(state=None):
    """Get list of all registered workflow actions.

    :param state: filter actions by given item state if provided
    """
    if state is None:
        return actions
    else:

        def is_go(action, state):
            if action["include_states"]:
                return state in action["include_states"]
            elif action["exclude_states"]:
                return state not in action["exclude_states"]

        return [action for action in actions if is_go(action, state)]


def get_privileged_actions(privileges):
    """Get the actions that are within the privileged list

    :param privileges:
    :return: list of allowed actions
    """
    allowed_action = []
    for action in actions:
        # only add the action if it has some privileges
        add = len(action["privileges"]) > 0
        for action_privilege in action["privileges"]:
            if action_privilege not in privileges.keys():
                add = False
            else:
                # make sure the privilege is allowed
                if not privileges[action_privilege] == 1:
                    add = False
        if add:
            allowed_action.append(action)
    return allowed_action


def workflow_state(name):
    """Register new workflow state.

    :param name: unique name of state
    """
    allowed_workflow_states.append(name)
    states.append({"name": name})


def is_workflow_state_transition_valid(action_name, state):
    """
    Checks if the state transition is valid or not. It's assumed that there are no duplicate actions.

    :param action_name: Action Name as defined while registering Workflow Action.
    :param state: Current State of the content
    :return: true if valid, False otherwise.
    """
    return action_name in [action["name"] for action in get_workflow_actions(state)]


def get_workflow_states():
    """Get list of all registered workflow states."""
    return states


def set_default_state(doc, state):
    """Sets default state for given doc.

    :param doc: item
    :param state: state to be set as default
    """
    doc.setdefault(ITEM_STATE, state)



# -*- coding: utf-8; -*-
#
# This file is part of Superdesk.
#
# Copyright 2013, 2014, 2015 Sourcefabric z.u. and contributors.
#
# For the full copyright and license information, please see the
# AUTHORS and LICENSE files distributed with this source code, or
# at https://www.sourcefabric.org/superdesk/license

import datetime
import lxml.etree as etree

from unittest import mock

from apps.publish import init_app
from superdesk.tests import TestCase
from superdesk.utc import utcnow
from superdesk.publish.formatters import NewsMLG2Formatter


NSMAP = {
    None: "http://iptc.org/std/nar/2006-10-01/",
    "xml": "http://www.w3.org/XML/1998/namespace",
}

XML_LANG = "{{{}}}lang".format(NSMAP["xml"])


def ns(key):
    tokens = key.split("/")
    return "/".join(["{http://iptc.org/std/nar/2006-10-01/}%s" % token for token in tokens])


@mock.patch("superdesk.publish.subscribers.SubscribersService.generate_sequence_number", lambda self, subscriber: 1)
class NewsMLG2FormatterTest(TestCase):
    embargo_ts = utcnow() + datetime.timedelta(days=2)
    article = {
        "guid": "tag:aap.com.au:20150613:12345",
        "_current_version": 1,
        "anpa_category": [{"qcode": "a", "name": "Australian General News"}],
        "source": "AAP",
        "headline": "This is a test headline - для теста!",
        "byline": "joe",
        "slugline": "slugline",
        "subject": [
            {"qcode": "02011001", "name": "international court or tribunal"},
            {"qcode": "02011002", "name": "extradition"},
        ],
        "anpa_take_key": "take_key",
        "unique_id": "1",
        "body_html": "The story body",
        "type": "text",
        "word_count": "1",
        "priority": "1",
        "_id": "urn:localhost.abc",
        "state": "published",
        "urgency": 2,
        "pubstatus": "usable",
        "dateline": {
            "source": "AAP",
            "text": "Los Angeles, Aug 11 AAP -",
            "located": {
                "alt_name": "",
                "state": "California",
                "city_code": "Los Angeles",
                "city": "Los Angeles",
                "dateline": "city",
                "country_code": "US",
                "country": "USA",
                "tz": "America/Los_Angeles",
                "state_code": "CA",
            },
        },
        "creditline": "sample creditline",
        "keywords": ["traffic"],
        "abstract": "sample abstract",
        "place": [
            {"qcode": "Australia", "name": "Australia", "state": "", "country": "Australia", "world_region": "Oceania"}
        ],
        "embargo": embargo_ts,
        "company_codes": [{"name": "YANCOAL AUSTRALIA LIMITED", "qcode": "YAL", "security_exchange": "ASX"}],
        "genre": [
            {"name": "Article (News)", "qcode": "article"},
            {"name": "foo", "code": "foo", "scheme": "custom"},
        ],
    }

    package = {
        "_id": "urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701",
        "type": "composite",
        "_current_version": 1,
        "groups": [
            {"role": "grpRole:NEP", "id": "root", "refs": [{"idRef": "main"}, {"idRef": "sidebar"}]},
            {
                "role": "grpRole:main",
                "id": "main",
                "refs": [
                    {
                        "type": "text",
                        "renditions": {},
                        "itemClass": "icls:text",
                        "guid": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "residRef": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "location": "archive",
                        "headline": "US:US cop sacked over student shooting",
                        "slugline": "US Police",
                    }
                ],
            },
            {
                "id": "sidebar",
                "refs": [{"type": "text", "itemClass": "icls:text", "headline": "Sidebar", "location": "archive"}],
            },
        ],
        "pubstatus": "usable",
        "state": "published",
        "flags": {"marked_for_not_publication": False},
        "guid": "urn:newsml:localhost:2015-08-12T11:59:58.457029:7e90d257-92f6-406d-9186-95653b211701",
        "dateline": {
            "located": {
                "alt_name": "",
                "state": "California",
                "city_code": "Los Angeles",
                "city": "Los Angeles",
                "dateline": "city",
                "country_code": "US",
                "country": "USA",
                "tz": "America/Los_Angeles",
                "state_code": "CA",
            },
            "date": "2015-08-12T01:59:58.000Z",
            "source": "AAP",
            "text": "Los Angeles, Aug 11 AAP -",
        },
        "language": "en",
        "headline": "Cop sacked over student shooting",
        "source": "AAP",
        "slugline": "US Police",
        "anpa_category": [{"name": "International News", "qcode": "I"}],
        "subject": [{"name": "police", "parent": "02000000", "qcode": "02003000"}],
    }

    picture_package = {
        "_id": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "guid": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "language": "en",
        "pubstatus": "usable",
        "groups": [
            {"refs": [{"idRef": "main"}], "id": "root", "role": "grpRole:NEP"},
            {
                "refs": [
                    {
                        "guid": "20150813001165688150",
                        "headline": "Prison Riot",
                        "residRef": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
                        "location": "archive",
                        "type": "picture",
                        "slugline": "Prison Riot",
                        "renditions": {
                            "baseImage": {
                                "height": 1400,
                                "mimetype": "image/jpeg",
                                "width": 1120,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650a",
                            },
                            "thumbnail": {
                                "height": 120,
                                "mimetype": "image/jpeg",
                                "width": 96,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650c",
                            },
                            "viewImage": {
                                "height": 640,
                                "mimetype": "image/jpeg",
                                "width": 512,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650e",
                            },
                            "original": {
                                "height": 800,
                                "mimetype": "image/jpeg",
                                "width": 640,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b6508",
                            },
                        },
                        "itemClass": "icls:picture",
                    }
                ],
                "id": "main",
                "role": "grpRole:main",
            },
        ],
        "type": "composite",
        "state": "published",
        "slugline": "Prison Riot",
        "description": "This Jan. 21, 2015 photo is of something)",
        "source": "AAP",
        "headline": "Prison Riot",
        "_current_version": 1,
        "dateline": {"date": "2015-08-13T04:07:59.000Z", "source": "AAP"},
        "flags": {"marked_for_not_publication": False},
        "sign_off": "mar",
    }

    picture = {
        "_id": "tag:localhost:2015:cf15b059-b997-4e34-a103-85b8d7ea4ba3",
        "dateline": {"source": "AAP", "date": "2015-08-14T04:45:53.000Z"},
        "renditions": {
            "viewImage": {
                "height": 415,
                "href": "http://localhost:5000/api/upload/55cd72811d41c828e1773786/raw?_schema=http",
                "media": "55cd72811d41c828e1773786",
                "mimetype": "image/jpeg",
                "width": 640,
            },
            "baseImage": {
                "height": 909,
                "href": "http://localhost:5000/api/upload/55cd72811d41c828e1773782/raw?_schema=http",
                "media": "55cd72811d41c828e1773782",
                "mimetype": "image/jpeg",
                "width": 1400,
            },
            "thumbnail": {
                "height": 120,
                "href": "http://localhost:5000/api/upload/55cd72811d41c828e1773784/raw?_schema=http",
                "media": "55cd72811d41c828e1773784",
                "mimetype": "image/jpeg",
                "width": 184,
            },
            "original": {
                "height": 2455,
                "href": "http://localhost:5000/api/upload/55cd72801d41c828e1773762/raw?_schema=http",
                "media": "55cd72801d41c828e1773762",
                "mimetype": "image/jpeg",
                "width": 3777,
            },
        },
        "state": "published",
        "guid": "20150731001161435160",
        "source": "AAP Image",
        "_current_version": 1,
        "original_source": "AAP Image/AAP",
        "description": "Federal Education Minister Christopher Pyne launches his new book NO ARCHIVING",
        "type": "picture",
        "slugline": "NUS CHRISTOPHER PYNE PROTEST",
        "headline": "NUS CHRISTOPHER PYNE PROTEST",
        "pubstatus": "usable",
        "ednote": "",
        "byline": "TRACEY NEARMY",
        "filemeta": {
            "yresolution": [300, 1],
            "exposuretime": [1, 200],
            "copyright": "                                                      ",
            "scenecapturetype": 0,
            "sensingmethod": 2,
            "fnumber": [14, 5],
            "flashpixversion": "0100",
            "xresolution": [300, 1],
            "resolutionunit": 2,
            "subsectimedigitized": "20",
            "exposureprogram": 1,
            "subsectimeoriginal": "20",
            "make": "NIKON CORPORATION",
            "focallengthin35mmfilm": 200,
            "scenetype": 1,
            "exifimageheight": 2455,
            "saturation": 0,
            "colorspace": 1,
            "subjectdistancerange": 0,
            "datetime": "2015:07:31 18:55:37",
            "software": "Photogene for iPad v4.3",
            "flash": 16,
            "focallength": [200, 1],
            "componentsconfiguration": "\u0001\u0002\u0003\u0000",
            "lightsource": 3,
            "artist": "                                    ",
            "isospeedratings": 2000,
            "whitepoint": [313, 1000],
            "sharpness": 2,
            "exposuremode": 1,
            "meteringmode": 3,
            "compressedbitsperpixel": [4, 1],
            "model": "NIKON D800E",
            "subsectime": "20",
            "datetimedigitized": "2015:07:31 18:55:37",
            "exifoffset": 406,
            "contrast": 0,
            "whitebalance": 1,
            "exifimagewidth": 3777,
            "datetimeoriginal": "2015:07:31 18:55:37",
            "customrendered": 0,
            "maxaperturevalue": [3, 1],
            "digitalzoomratio": [1, 1],
            "primarychromaticities": [16, 25],
            "length": 8009209,
            "exifversion": "0230",
            "gaincontrol": 2,
            "gamma": [11, 5],
            "filesource": 3,
        },
        "language": "en",
        "mimetype": "image/jpeg",
        "sign_off": "mar",
        "unique_id": 573,
    }

    video = {
        "_id": "urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e",
        "guid": "tag:localhost:2015:c11e11c4-cdbc-41ef-b939-2b30dd8365fb",
        "language": "en",
        "family_id": "urn:newsml:localhost:2015-09-20T16:12:57.333001:f3856812-0999-4ed8-b69e-68dcdeb1ed2e",
        "_current_version": 3,
        "versioncreated": "2015-09-20T06:14:11.000Z",
        "unique_id": 274,
        "renditions": {
            "original": {
                "media": "55fe4e691d41c8cac923ceb2",
                "href": "http://192.168.220.176:5000/api/upload/55fe4e691d41c8cac923ceb2/raw?_schema=http",
                "mimetype": "video/mp4",
            }
        },
        "state": "in_progress",
        "version_creator": "55ee82871d41c86ee1d78c45",
        "sign_off": "ADM",
        "media": "55fe4e691d41c8cac923ceb2",
        "source": "AAP",
        "original_source": "AAP Video/AAP",
        "pubstatus": "usable",
        "filemeta": {
            "mime_type": "video/mp4",
            "last_modification": "1904-01-01T00:00:00+00:00",
            "creation_date": "1904-01-01T00:00:00+00:00",
            "height": "270",
            "width": "480",
            "duration": "0:00:10.224000",
            "comment": "User volume: 100.0%",
            "length": 877869,
            "endian": "Big endian",
        },
        "event_id": "tag:localhost:2015:f3ae4441-4721-4987-8265-88d747b6a550",
        "original_creator": "55ee82871d41c86ee1d78c45",
        "expiry": "2016-12-21T14:14:11.000Z",
        "firstcreated": "2015-09-20T06:12:57.000Z",
        "_created": "2015-09-20T06:12:57.000Z",
        "type": "video",
        "unique_name": "#274",
        "mimetype": "video/mp4",
        "version": 2,
        "headline": "test",
        "description": "testing video",
        "abstract": "test video",
        "slugline": "test video",
        "byline": "test video",
        "subject": [{"qcode": "01001000", "name": "archaeology", "parent": "01000000"}],
        "place": [{"qcode": "ACT", "name": "ACT"}],
        "anpa_category": [{"qcode": "a", "name": "Australian General News"}],
    }

    picture_text_package = {
        "_id": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "guid": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "language": "en",
        "pubstatus": "usable",
        "groups": [
            {"refs": [{"idRef": "main"}], "id": "root", "role": "grpRole:NEP"},
            {
                "refs": [
                    {
                        "type": "text",
                        "renditions": {},
                        "itemClass": "icls:text",
                        "guid": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "residRef": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "location": "archive",
                        "headline": "US:US cop sacked over student shooting",
                        "slugline": "US Police",
                        "_current_version": 4,
                    },
                    {
                        "guid": "20150813001165688150",
                        "headline": "Prison Riot",
                        "residRef": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
                        "location": "archive",
                        "type": "picture",
                        "slugline": "Prison Riot",
                        "renditions": {
                            "baseImage": {
                                "height": 1400,
                                "mimetype": "image/jpeg",
                                "width": 1120,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650a",
                            },
                            "thumbnail": {
                                "height": 120,
                                "mimetype": "image/jpeg",
                                "width": 96,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650c",
                            },
                            "viewImage": {
                                "height": 640,
                                "mimetype": "image/jpeg",
                                "width": 512,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650e",
                            },
                            "original": {
                                "height": 800,
                                "mimetype": "image/jpeg",
                                "width": 640,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b6508",
                            },
                        },
                        "itemClass": "icls:picture",
                        "_current_version": 4,
                    },
                ],
                "id": "main",
                "role": "grpRole:main",
            },
        ],
        "type": "composite",
        "state": "published",
        "slugline": "Prison Riot",
        "description": "This Jan. 21, 2015 photo is of something)",
        "source": "AAP",
        "headline": "Prison Riot",
        "_current_version": 1,
        "dateline": {"date": "2015-08-13T04:07:59.000Z", "source": "AAP"},
        "flags": {"marked_for_not_publication": False},
        "sign_off": "mar",
    }

    picture_text_package_multi_group = {
        "_id": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "guid": "urn:newsml:localhost:2015-08-13T14:07:59.846466:c659e21b-1ea2-48b7-9b35-e971ae9d1e6e",
        "language": "en",
        "pubstatus": "usable",
        "groups": [
            {"refs": [{"idRef": "main"}, {"idRef": "picture"}], "id": "root", "role": "grpRole:NEP"},
            {
                "refs": [
                    {
                        "type": "text",
                        "renditions": {},
                        "itemClass": "icls:text",
                        "guid": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "residRef": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
                        "location": "archive",
                        "headline": "US:US cop sacked over student shooting",
                        "slugline": "US Police",
                        "_current_version": 4,
                    }
                ],
                "id": "main",
                "role": "grpRole:main",
            },
            {
                "refs": [
                    {
                        "guid": "20150813001165688150",
                        "headline": "Prison Riot",
                        "residRef": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
                        "location": "archive",
                        "type": "picture",
                        "slugline": "Prison Riot",
                        "renditions": {
                            "baseImage": {
                                "height": 1400,
                                "mimetype": "image/jpeg",
                                "width": 1120,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650a/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650a",
                            },
                            "thumbnail": {
                                "height": 120,
                                "mimetype": "image/jpeg",
                                "width": 96,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650c/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650c",
                            },
                            "viewImage": {
                                "height": 640,
                                "mimetype": "image/jpeg",
                                "width": 512,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b650e/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b650e",
                            },
                            "original": {
                                "height": 800,
                                "mimetype": "image/jpeg",
                                "width": 640,
                                "href": "http://localhost:5000/api/upload/55cc03731d41c8cea12b6508/raw?_schema=http",
                                "media": "55cc03731d41c8cea12b6508",
                            },
                        },
                        "itemClass": "icls:picture",
                        "_current_version": 4,
                    }
                ],
                "id": "picture",
                "role": "grpRole:picture",
            },
        ],
        "type": "composite",
        "state": "published",
        "slugline": "Prison Riot",
        "description": "This Jan. 21, 2015 photo is of something)",
        "source": "AAP",
        "headline": "Prison Riot",
        "_current_version": 1,
        "dateline": {"date": "2015-08-13T04:07:59.000Z", "source": "AAP"},
        "flags": {"marked_for_not_publication": False},
        "sign_off": "mar",
    }

    packaged_articles = [
        {
            "_id": "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
            "headline": "package article headline",
            "slugline": "slugline",
            "pubStatus": "usable",
        },
        {
            "_id": "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
            "headline": "package article headline",
            "slugline": "slugline",
            "pubStatus": "usable",
        },
    ]

    vocab = [
        {
            "_id": "rightsinfo",
            "items": [
                {
                    "name": "AAP",
                    "copyrightHolder": "copy right holder",
                    "copyrightNotice": "copy right notice",
                    "usageTerms": "terms",
                },
                {
                    "name": "default",
                    "copyrightHolder": "default copy right holder",
                    "copyrightNotice": "default copy right notice",
                    "usageTerms": "default terms",
                },
            ],
        }
    ]

    now = datetime.datetime(2015, 6, 13, 11, 45, 19, 0)

    def setUp(self):
        self.article["state"] = "published"
        self.article["firstcreated"] = self.now
        self.article["versioncreated"] = self.now
        self.newsml = etree.Element("NewsML")
        self.formatter = NewsMLG2Formatter()
        self.formatter.now = self.now
        self.formatter.string_now = self.now.strftime("%Y-%m-%dT%H:%M:%S.0000Z")
        init_app(self.app)
        self.app.data.insert("vocabularies", self.vocab)
        self.app.data.insert("archive", self.packaged_articles)

    def test_formatter(self):
        seq, doc = self.formatter.format(self.article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        content_meta = xml.find(ns("itemSet")).find(ns("newsItem")).find(ns("contentMeta"))
        self.assertEqual(
            xml.find("{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}sender").text,
            "sourcefabric.org",
        )
        self.assertEqual(
            xml.find("{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority").text,
            "1",
        )
        self.assertEqual(
            xml.find("{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}origin").text,
            "AAP",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}rightsInfo/{http://iptc.org/std/nar/2006-10-01/}usageTerms"
            ).text,
            "terms",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}itemMeta/{http://iptc.org/std/nar/2006-10-01/}provider/"
                + "{http://iptc.org/std/nar/2006-10-01/}name"
            ).text,
            "sourcefabric.org",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/{http://iptc.org/std/nar/2006-10-01/}headline"
            ).text,
            "This is a test headline - для теста!",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/{http://iptc.org/std/nar/2006-10-01/}urgency"
            ).text,
            "2",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/{http://iptc.org/std/nar/2006-10-01/}dateline"
            ).text,
            "Los Angeles, Aug 11 AAP -",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/"
                + '{http://iptc.org/std/nar/2006-10-01/}located[@qcode="loctyp:City"]/'
                + "{http://iptc.org/std/nar/2006-10-01/}name"
            ).text,
            "Los Angeles",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/"
                + "{http://iptc.org/std/nar/2006-10-01/}located/"
                + '{http://iptc.org/std/nar/2006-10-01/}broader[@qcode="loctyp:CountryArea"]/'
                + "{http://iptc.org/std/nar/2006-10-01/}name"
            ).text,
            "California",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/"
                + "{http://iptc.org/std/nar/2006-10-01/}located/"
                + '{http://iptc.org/std/nar/2006-10-01/}broader[@qcode="loctyp:Country"]/'
                + "{http://iptc.org/std/nar/2006-10-01/}name"
            ).text,
            "USA",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/{http://iptc.org/std/nar/2006-10-01/}inlineXML/"
                + "{http://iptc.org/std/nar/2006-10-01/}nitf/{http://iptc.org/std/nar/2006-10-01/}body/"
                + "{http://iptc.org/std/nar/2006-10-01/}body.content/"
                + "{http://iptc.org/std/nar/2006-10-01/}p"
            ).text,
            "The story body",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}itemMeta/{http://iptc.org/std/nar/2006-10-01/}embargoed"
            ).text,
            self.embargo_ts.isoformat(),
        )

        company_details = xml.find(
            "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
            + "{http://iptc.org/std/nar/2006-10-01/}assert/{http://iptc.org/std/nar/2006-10-01/}organisationDetails/"
            + "{http://iptc.org/std/nar/2006-10-01/}hasInstrument[@symbol]"
        )
        self.assertEqual(company_details.attrib.get("symbol", ""), "YAL")
        self.assertEqual(company_details.attrib.get("marketlabel", ""), "ASX")

        company_details = xml.find(
            "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
            + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/"
            + '{http://iptc.org/std/nar/2006-10-01/}subject[@type="cpnat:organisation"]/'
            + "{http://iptc.org/std/nar/2006-10-01/}name"
        )
        self.assertEqual(company_details.text, "YANCOAL AUSTRALIA LIMITED")

        genre = content_meta.findall(ns("genre"))
        self.assertEqual(2, len(genre))
        self.assertEqual("article", genre[0].get("qcode"))
        self.assertEqual("Article (News)", genre[0].find(ns("name")).text)
        self.assertEqual("custom:foo", genre[1].get("qcode"))

    def testPreservedFomat(self):
        article = dict(self.article)
        article["format"] = "preserved"
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/{http://iptc.org/std/nar/2006-10-01/}inlineData"
            ).text,
            "The story body",
        )

    def testDefaultRightsFomatter(self):
        article = dict(self.article)
        article["source"] = "BOGUS"
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}rightsInfo/{http://iptc.org/std/nar/2006-10-01/}usageTerms"
            ).text,
            "default terms",
        )

    def testPackagePublish(self):
        article = dict(self.package)
        article["firstcreated"] = self.now
        article["versioncreated"] = self.now
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        self.assertEqual(
            xml.find("{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority").text,
            "5",
        )

        groups = xml.findall(ns("itemSet/packageItem/groupSet/group"))
        self.assertEqual(3, len(groups))

        self.assertEqual(xml.find(ns("itemSet/packageItem/groupSet/group/itemRef/slugline")).text, "slugline")
        self.assertEqual(
            xml.find(ns("itemSet/packageItem/groupSet/group/itemRef")).get("residref"),
            "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b",
        )

    def testPicturePackagePublish(self):
        article = dict(self.picture_package)
        article["firstcreated"] = self.now
        article["versioncreated"] = self.now
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        self.assertEqual(
            xml.find("{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority").text,
            "5",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}packageItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}groupSet/{http://iptc.org/std/nar/2006-10-01/}group/"
                + "{http://iptc.org/std/nar/2006-10-01/}itemRef/{http://iptc.org/std/nar/2006-10-01/}slugline"
            ).text,
            "slugline",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}packageItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}groupSet/{http://iptc.org/std/nar/2006-10-01/}group/"
                + "{http://iptc.org/std/nar/2006-10-01/}itemRef"
            ).get("residref"),
            "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037",
        )

    def testPicturePublish(self):
        article = dict(self.picture)
        article["firstcreated"] = self.now
        article["versioncreated"] = self.now
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        self.assertEqual(
            xml.find("{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority").text,
            "5",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/"
                + "{http://iptc.org/std/nar/2006-10-01/}creditline"
            ).text,
            "AAP Image/AAP",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/"
                + '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]'
            ).get("width"),
            "3777",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/"
                + '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]'
            ).get("height"),
            "2455",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/"
                + '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]'
            ).get("contenttype"),
            "image/jpeg",
        )

    def testVideoPublish(self):
        article = dict(self.video)
        article["firstcreated"] = self.now
        article["versioncreated"] = self.now
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        self.assertEqual(
            xml.find("{http://iptc.org/std/nar/2006-10-01/}header/{http://iptc.org/std/nar/2006-10-01/}priority").text,
            "5",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentMeta/"
                + "{http://iptc.org/std/nar/2006-10-01/}creditline"
            ).text,
            "AAP Video/AAP",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/"
                + '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]'
            ).get("width"),
            "480",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/"
                + '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]'
            ).get("height"),
            "270",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/"
                + '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]'
            ).get("duration"),
            "0:00:10.224000",
        )
        self.assertEqual(
            xml.find(
                "{http://iptc.org/std/nar/2006-10-01/}itemSet/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
                + "{http://iptc.org/std/nar/2006-10-01/}contentSet/"
                + '{http://iptc.org/std/nar/2006-10-01/}remoteContent[@rendition="rendition:original"]'
            ).get("contenttype"),
            "video/mp4",
        )

    def testPictureTextPackage(self):
        article = dict(self.picture_text_package)
        article["firstcreated"] = self.now
        article["versioncreated"] = self.now
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        item_refs = xml.findall(".//{http://iptc.org/std/nar/2006-10-01/}itemRef")
        self.assertEqual(len(item_refs), 2)
        self.assertEqual(item_refs[0].get("residref"), "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b")
        self.assertEqual(item_refs[0].find("{http://iptc.org/std/nar/2006-10-01/}itemClass").get("qcode"), "ninat:text")
        self.assertEqual(
            item_refs[0].find("{http://iptc.org/std/nar/2006-10-01/}pubStatus").get("qcode"), "stat:usable"
        )
        self.assertEqual(item_refs[1].get("residref"), "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037")
        self.assertEqual(
            item_refs[1].find("{http://iptc.org/std/nar/2006-10-01/}itemClass").get("qcode"), "ninat:picture"
        )
        self.assertEqual(
            item_refs[1].find("{http://iptc.org/std/nar/2006-10-01/}pubStatus").get("qcode"), "stat:usable"
        )

    def testPictureTextPackageMultiGroup(self):
        article = dict(self.picture_text_package_multi_group)
        article["firstcreated"] = self.now
        article["versioncreated"] = self.now
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        item_refs = xml.findall(".//{http://iptc.org/std/nar/2006-10-01/}itemRef")
        self.assertEqual(len(item_refs), 2)
        self.assertEqual(item_refs[0].get("residref"), "tag:localhost:2015:5838657b-b3ec-4e5a-9b39-36039e16400b")
        self.assertEqual(item_refs[0].find("{http://iptc.org/std/nar/2006-10-01/}itemClass").get("qcode"), "ninat:text")
        self.assertEqual(
            item_refs[0].find("{http://iptc.org/std/nar/2006-10-01/}pubStatus").get("qcode"), "stat:usable"
        )
        self.assertEqual(item_refs[1].get("residref"), "tag:localhost:2015:0c12aa0a-82ef-4c58-a363-c5bd8a368037")
        self.assertEqual(
            item_refs[1].find("{http://iptc.org/std/nar/2006-10-01/}itemClass").get("qcode"), "ninat:picture"
        )
        self.assertEqual(
            item_refs[1].find("{http://iptc.org/std/nar/2006-10-01/}pubStatus").get("qcode"), "stat:usable"
        )
        groups = xml.findall(".//{http://iptc.org/std/nar/2006-10-01/}group")
        self.assertEqual(len(groups), 3)
        self.assertEqual(groups[0].get("id"), "root")
        self.assertEqual(groups[0].get("role"), "grpRole:NEP")
        self.assertEqual(groups[1].get("id"), "main")
        self.assertEqual(groups[1].get("role"), "grpRole:main")
        self.assertEqual(groups[2].get("id"), "picture")
        self.assertEqual(groups[2].get("role"), "grpRole:picture")

        group_ref = xml.findall(".//{http://iptc.org/std/nar/2006-10-01/}groupRef")
        self.assertEqual(len(group_ref), 2)
        self.assertEqual(group_ref[0].get("idref"), "main")
        self.assertEqual(group_ref[1].get("idref"), "picture")

    def testPlace(self):
        article = self.article.copy()
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        content_meta = xml.find(
            "{http://iptc.org/std/nar/2006-10-01/}itemSet"
            "/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
            "{http://iptc.org/std/nar/2006-10-01/}contentMeta"
        )
        subject = content_meta.find('{http://iptc.org/std/nar/2006-10-01/}subject[@qcode="loctyp:Country"]')
        self.assertEqual(subject.find("{http://iptc.org/std/nar/2006-10-01/}name").text, "Australia")
        self.assertEqual(
            subject.find(
                "{http://iptc.org/std/nar/2006-10-01/}broader"
                + '[@qcode="loctyp:WorldArea"]/'
                + "{http://iptc.org/std/nar/2006-10-01/}name"
            ).text,
            "Oceania",
        )
        self.assertIsNone(
            content_meta.find("{http://iptc.org/std/nar/2006-10-01/}" 'subject[@qcode="loctyp:CountryArea"]')
        )
        article["place"] = [
            {
                "name": "ACT",
                "qcode": "ACT",
                "state": "Australian Capital Territory",
                "country": "Australia",
                "world_region": "Oceania",
            }
        ]
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        content_meta = xml.find(
            "{http://iptc.org/std/nar/2006-10-01/}itemSet"
            "/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
            "{http://iptc.org/std/nar/2006-10-01/}contentMeta"
        )
        subject = content_meta.find('{http://iptc.org/std/nar/2006-10-01/}subject[@qcode="loctyp:CountryArea"]')
        self.assertEqual(subject.find("{http://iptc.org/std/nar/2006-10-01/}name").text, "Australian Capital Territory")
        self.assertEqual(
            subject.find(
                "{http://iptc.org/std/nar/2006-10-01/}broader"
                + '[@qcode="loctyp:Country"]/'
                + "{http://iptc.org/std/nar/2006-10-01/}name"
            ).text,
            "Australia",
        )
        self.assertEqual(
            subject.find(
                "{http://iptc.org/std/nar/2006-10-01/}broader"
                + '[@qcode="loctyp:WorldArea"]/'
                + "{http://iptc.org/std/nar/2006-10-01/}name"
            ).text,
            "Oceania",
        )

        article["place"] = [{"name": "EUR", "qcode": "EUR", "state": "", "country": "", "world_region": "Europe"}]
        seq, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        content_meta = xml.find(
            "{http://iptc.org/std/nar/2006-10-01/}itemSet"
            "/{http://iptc.org/std/nar/2006-10-01/}newsItem/"
            "{http://iptc.org/std/nar/2006-10-01/}contentMeta"
        )
        subject = content_meta.find('{http://iptc.org/std/nar/2006-10-01/}subject[@qcode="loctyp:WorldArea"]')
        self.assertEqual(subject.find("{http://iptc.org/std/nar/2006-10-01/}name").text, "Europe")
        self.assertIsNone(
            content_meta.find("{http://iptc.org/std/nar/2006-10-01/}" 'subject[@qcode="loctyp:CountryArea"]')
        )
        self.assertIsNone(content_meta.find("{http://iptc.org/std/nar/2006-10-01/}" 'subject[@qcode="loctyp:Country"]'))

    def testPlaceGeonames(self):
        article = self.article.copy()
        article["place"] = [
            {
                "name": "Kobeřice",
                "code": "3073493",
                "scheme": "geonames",
                "state": "Moravskoslezský",
                "country": "Czechia",
                "state_code": "80",
                "country_code": "CZ",
                "feature_class": "P",
                "location": {
                    "lat": 49.98548,
                    "lon": 18.05212,
                },
            },
            {
                "name": "Moravskoslezsky",
                "code": "3339573",
                "scheme": "geonames",
                "state": "Moravskoslezský",
                "country": "Czechia",
                "state_code": "80",
                "country_code": "CZ",
                "feature_class": "A",
                "location": {
                    "lat": 49.83333,
                    "lon": 18.0,
                },
            },
            {
                "name": "Czechia",
                "code": "3077311",
                "scheme": "geonames",
                "state": "",
                "state_code": None,
                "country": "Czechia",
                "country_code": "CZ",
                "feature_class": "A",
                "location": {
                    "lat": 49.75,
                    "lon": 15.0,
                },
            },
        ]

        _, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        xml = etree.fromstring(doc.encode("utf-8"))
        content_meta = xml.find(ns("itemSet")).find(ns("newsItem")).find(ns("contentMeta"))

        # test city
        subject = content_meta.find(ns('subject[@type="cpnat:geoArea"]'))
        self.assertIsNotNone(subject, etree.tostring(content_meta, pretty_print=True))
        self.assertEqual(article["place"][0]["name"], subject.find(ns("name")).text)

        broader = subject.findall(ns("broader"))
        self.assertEqual(2, len(broader))
        self.assertEqual("iso3166-2:CZ-80", broader[0].get("qcode"))
        self.assertEqual("cptype:statprov", broader[0].get("type"))
        self.assertEqual(article["place"][0]["state"], broader[0].find(ns("name")).text)
        self.assertEqual("iso3166-1a2:CZ", broader[1].get("qcode"))
        self.assertEqual("cptype:country", broader[1].get("type"))
        self.assertEqual(article["place"][0]["country"], broader[1].find(ns("name")).text)

        geo_area = subject.find(ns("geoAreaDetails"))
        self.assertIsNotNone(geo_area)
        position = geo_area.find(ns("position"))
        self.assertIsNotNone(position)
        self.assertEqual("49.98548", position.get("latitude"))
        self.assertEqual("18.05212", position.get("longitude"))

        subjects = content_meta.findall(ns('subject[@type="cpnat:geoArea"]'))
        self.assertEqual(3, len(subjects))

        # test state
        state = subjects[1]
        self.assertEqual("Moravskoslezsky", state.find(ns("name")).text)
        self.assertEqual("iso3166-2:CZ-80", state.get("qcode"))
        broader = state.findall(ns("broader"))
        self.assertEqual(1, len(broader))
        self.assertEqual("iso3166-1a2:CZ", broader[0].get("qcode"))
        self.assertEqual("cptype:country", broader[0].get("type"))
        self.assertEqual("Czechia", broader[0].find(ns("name")).text)

        # test country
        country = subjects[2]
        self.assertEqual("Czechia", country.find(ns("name")).text)
        self.assertEqual("iso3166-1a2:CZ", country.get("qcode"))
        broader = country.findall(ns("broader"))
        self.assertEqual(0, len(broader))

    def format(self, updates=None):
        article = self.article.copy()
        article.update(updates)
        _, doc = self.formatter.format(article, {"name": "Test Subscriber"})[0]
        root = etree.fromstring(doc.encode("utf-8"))
        item = root.find("itemSet", NSMAP).find("newsItem", NSMAP)
        return item

    def test_lang_fr(self):
        item = self.format(
            {
                "language": "fr-CA",
                "subject": [
                    {
                        "name": "foo",
                        "qcode": "foo",
                        "translations": {
                            "name": {"fr": "foo fr", "en": "foo en"},
                        },
                    },
                    {"name": "bar", "qcode": "bar"},
                ],
                "anpa_category": [
                    {
                        "name": "cat",
                        "qcode": "cat",
                        "translations": {
                            "name": {"fr-CA": "cat fr", "en": "cat en"},
                        },
                    },
                ],
                "place": [],
                "company_codes": [],
                "genre": [
                    {
                        "name": "gen",
                        "qcode": "gen",
                        "translations": {
                            "name": {"fr": "gen fr", "en": "gen en"},
                        },
                    },
                ],
            }
        )

        self.assertIsNotNone(item)
        self.assertEqual("fr-CA", item.attrib.get(XML_LANG))

        meta = item.find("contentMeta", NSMAP)
        self.assertIsNotNone(meta)

        subject = meta.findall("subject", NSMAP)
        self.assertEqual(3, len(subject))
        self.assertEqual("foo fr", subject[0].find("name", NSMAP).text)
        self.assertEqual("fr", subject[0].find("name", NSMAP).attrib[XML_LANG])
        self.assertEqual("bar", subject[1].find("name", NSMAP).text)
        self.assertEqual("en", subject[1].find("name", NSMAP).attrib[XML_LANG])
        self.assertEqual("cat fr", subject[2].find("name", NSMAP).text)
        self.assertEqual("fr-CA", subject[2].find("name", NSMAP).attrib[XML_LANG])

        genre = meta.find("genre", NSMAP)
        self.assertEqual("gen fr", genre.find("name", NSMAP).text)
        self.assertEqual("fr", genre.find("name", NSMAP).attrib[XML_LANG])
